__all__ = 'APISchema'

import typing

from . import constants
from .plugin_accessor import PluginAccessor
from .errors.rejected_transition_input_error import RejectedTransitionInputError
from .inference_source import InferenceSource

if typing.TYPE_CHECKING:
    from .attribute_types import EnumAttributeType
    from .job_definition import JobDefinition
    from .submit2gm_plugin import Submit2gmPlugin


class APISchema(PluginAccessor):

    def __init__(self,
                 plugin: 'Submit2gmPlugin',
                 job_definitions: typing.List['JobDefinition']):
        PluginAccessor.__init__(self, plugin)
        self._job_definitions = job_definitions

    def get_job_definitions(self) -> typing.List['JobDefinition']:
        return self._job_definitions

    def get_job_definition_with_id(self, identifier: str) -> 'JobDefinition':

        for job_definition in self.get_job_definitions():
            if job_definition.get_definition_id() == identifier:
                return job_definition

        raise ValueError("No job definition with identifier '" + identifier + "'")
