__all__ = 'CachedValue'

import copy
import typing

T = typing.TypeVar('T')


class CachedValue(typing.Generic[T]):
    def __init__(self,
                 value: T = None,
                 is_cached: bool = False):
        self._initial_value = copy.copy(value)
        self._value = value
        self._is_cached = is_cached

    def get_value(self) -> T:
        return self._value

    def set_value(self, value: T):
        self._value = value
        self._is_cached = True

    def is_cached(self) -> bool:
        return self._is_cached

    def reset_and_clear_cache(self) -> None:
        self._value = copy.copy(self._initial_value)
        self._is_cached = False
