import enum
import math
import pathlib

PROGRAM_NAME = "Submit2GM"
APP_DESCRIPTION = ""
BANNER_URL = "https://www.gridmarkets.com/contact-us"
HELP_URL = "https://www.gridmarkets.com/faq"

ASSETS_DIRECTORY = pathlib.Path(__file__).parent.parent / pathlib.Path("assets")
FONTS_DIRECTORY = ASSETS_DIRECTORY / pathlib.Path("fonts")
ICONS_DIRECTORY = ASSETS_DIRECTORY / pathlib.Path("icons")
IMAGES_DIRECTORY = ASSETS_DIRECTORY / pathlib.Path("images")

JOB_DEFINITION_ID_FLAG = '--job-definition-id'
CLI_LOGGER_ID = 'cli'

COMMAND_PORT_SERVER_TIMEOUT = (10, 10)  # (connect, read)
COMMAND_PORT_SERVER_TIMEOUT_KEEP_ALIVE_TOKEN = b'_'

FILE_SYSTEM_WATCHER_PATH = pathlib.Path(__file__).parent.parent

APPLICATION_LINKING_TOOL_TIP = "Toggle to enable / disable application linking for this attribute. " \
                               "Enabling will sync this attribute's value from your application."

class SUBMIT2GM_CLI_ARGUMENTS(str, enum.Enum):
    SUBMIT_PROJECT = 'submit-project'
    PROJECT_REQUEST = '--project-request'
    USE_FILE = '--use-file'
    PROJECT_NAME = '--project-name'


FORBIDDEN_LINUX_PATHS = ["/proc", "/dev", "/usr", "/var", "/srv", "/sbin", "/opt", "/run", "/root", "/selinux", "/bin",
                         "/data", "/lib", "/etc", "/sys", "/tmp", "/home/gmadmin", "/home/gridmarkets", "/local"]


""" """
REMOTE_SOURCE_SUFFIX = "_remote"

# asset paths
SPINNER_GIF_PATH = ICONS_DIRECTORY / "spinner.gif"
SPINNER_WHITE_GIF_PATH = ICONS_DIRECTORY / "spinner_white.gif"
SPINNER_WHITE_2D2D2D_GIF_PATH = ICONS_DIRECTORY / "spinner_white_2d2d2d.gif"
SPINNER_WHITE_666666_GIF_PATH = ICONS_DIRECTORY / "spinner_white_666666.gif"
SPINNER_LARGE_GIF_PATH = ICONS_DIRECTORY / "spinner_large.gif"
LOADING_BAR_GIF_PATH = ICONS_DIRECTORY / "loading_bar.gif"

CHECKMARK_ICON_WHITE_PATH = ICONS_DIRECTORY / "checkmark_icon_white.png"
CHECKMARK_ICON_BLACK_PATH = ICONS_DIRECTORY / "checkmark_icon_black.png"
CHECKMARK_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "checkmark_icon_off_white.png"

DEPENDENCY_VIEW_ICON_BRAND_PATH = ICONS_DIRECTORY / "dependency_view_icon_brand.png"

DIRECTORY_ICON_WHITE_PATH = ICONS_DIRECTORY / "directory_icon_white.png"
DIRECTORY_ICON_BLACK_PATH = ICONS_DIRECTORY / "directory_icon_black.png"
DIRECTORY_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "directory_icon_off_white.png"

DIRECTORY_OUTLINE_ICON_WHITE_PATH = ICONS_DIRECTORY / "directory_outline_icon_white.png"
DIRECTORY_OUTLINE_ICON_BLACK_PATH = ICONS_DIRECTORY / "directory_outline_icon_black.png"
DIRECTORY_OUTLINE_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "directory_outline_icon_off_white.png"

DIRECTORY_HIDDEN_ICON_WHITE_PATH = ICONS_DIRECTORY / "directory_hidden_icon_white.png"
DIRECTORY_HIDDEN_ICON_BLACK_PATH = ICONS_DIRECTORY / "directory_hidden_icon_black.png"
DIRECTORY_HIDDEN_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "directory_hidden_icon_off_white.png"

EYE_ICON_WHITE_PATH = ICONS_DIRECTORY / "eye_icon_white.png"
EYE_ICON_BLACK_PATH = ICONS_DIRECTORY / "eye_icon_black.png"
EYE_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "eye_icon_off_white.png"

GM_ICON_WHITE_PATH = ICONS_DIRECTORY / "gm_icon_white.png"
GM_ICON_BLACK_PATH = ICONS_DIRECTORY / "gm_icon_black.png"
GM_ICON_BLACK_AND_WHITE_PATH = ICONS_DIRECTORY / "gm_icon_black_and_white.png"
GM_MAYA_ICON_PATH = ICONS_DIRECTORY / "gm_maya_icon.png"

USER_ICON_WHITE_PATH = ICONS_DIRECTORY / "user_icon_white.png"
USER_ICON_BLACK_PATH = ICONS_DIRECTORY / "user_icon_black.png"
USER_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "user_icon_off_white.png"

TRASH_ICON_WHITE_PATH = ICONS_DIRECTORY / "trash_icon_white.png"
TRASH_ICON_BLACK_PATH = ICONS_DIRECTORY / "trash_icon_black.png"
TRASH_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "trash_icon_off_white.png"

MENU_ICON_WHITE_PATH = ICONS_DIRECTORY / "menu_icon_white.png"
MENU_ICON_BLACK_PATH = ICONS_DIRECTORY / "menu_icon_black.png"
MENU_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "menu_icon_off_white.png"

NEW_JOB_PRESET_ICON_WHITE_PATH = ICONS_DIRECTORY / "new_job_preset_icon_white.png"
NEW_JOB_PRESET_ICON_BLACK_PATH = ICONS_DIRECTORY / "new_job_preset_icon_black.png"
NEW_JOB_PRESET_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "new_job_preset_icon_off_white.png"

PROJECT_SELECT_ICON_WHITE_PATH = ICONS_DIRECTORY / "project_select_icon_white.png"

LOCKED_ICON_WHITE_PATH = ICONS_DIRECTORY / "locked_icon_white.png"
LOCKED_ICON_BLACK_PATH = ICONS_DIRECTORY / "locked_icon_black.png"
LOCKED_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "locked_icon_off_white.png"

UNLOCKED_ICON_WHITE_PATH = ICONS_DIRECTORY / "unlocked_icon_white.png"
UNLOCKED_ICON_BLACK_PATH = ICONS_DIRECTORY / "unlocked_icon_black.png"
UNLOCKED_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "unlocked_icon_off_white.png"

INFO_ICON_WHITE_PATH = ICONS_DIRECTORY / "info_icon_white.png"
INFO_ICON_BLACK_PATH = ICONS_DIRECTORY / "info_icon_black.png"
INFO_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "info_icon_off_white.png"

FILE_ICON_WHITE_PATH = ICONS_DIRECTORY / "file_icon_white.png"
FILE_ICON_BLACK_PATH = ICONS_DIRECTORY / "file_icon_black.png"

FILE_OUTLINE_ICON_WHITE_PATH = ICONS_DIRECTORY / "file_outline_icon_white.png"
FILE_OUTLINE_ICON_BLACK_PATH = ICONS_DIRECTORY / "file_outline_icon_black.png"
FILE_OUTLINE_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "file_outline_icon_off_white.png"

NEW_FILE_ICON_WHITE_PATH = ICONS_DIRECTORY / "new_file_icon_white.png"
NEW_FILE_ICON_BLACK_PATH = ICONS_DIRECTORY / "new_file_icon_black.png"
NEW_FILE_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "new_file_icon_off_white.png"

COPY_ICON_WHITE_PATH = ICONS_DIRECTORY / "copy_icon_white.png"
COPY_ICON_BLACK_PATH = ICONS_DIRECTORY / "copy_icon_black.png"
COPY_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "copy_icon_off_white.png"

CPS_CONNECTION_ACTIVE_GREEN_PATH = ICONS_DIRECTORY / "cps_connection_active_green.png"

LINK_ICON_WHITE_PATH = ICONS_DIRECTORY / "link_icon_white.png"
LINK_ICON_DISABLED_WHITE_PATH = ICONS_DIRECTORY / "link_icon_disabled_white.png"
LINK_ICON_UNCHECKED_WHITE_PATH = ICONS_DIRECTORY / "link_icon_unchecked_white.png"
LINK_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "link_icon_off_white.png"
LINK_ICON_BRAND_PATH = ICONS_DIRECTORY / "link_icon_brand.png"
LINK_ICON_DISABLED_BRAND_PATH = ICONS_DIRECTORY / "link_icon_disabled_brand.png"

CONSOLE_ICON_WHITE_PATH = ICONS_DIRECTORY / "console_icon_white.png"
CONSOLE_ICON_BLACK_PATH = ICONS_DIRECTORY / "console_icon_black.png"
CONSOLE_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "console_icon_off_white.png"

UP_ARROW_ICON_WHITE_PATH = ICONS_DIRECTORY / "up_arrow_icon_white.png"
UP_ARROW_ICON_BLACK_PATH = ICONS_DIRECTORY / "up_arrow_icon_black.png"
UP_ARROW_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "up_arrow_icon_off_white.png"
UP_ARROW_ICON_OFF_BLACK_PATH = ICONS_DIRECTORY / "up_arrow_icon_off_black.png"

DOWN_ARROW_ICON_WHITE_PATH = ICONS_DIRECTORY / "down_arrow_icon_white.png"
DOWN_ARROW_ICON_BLACK_PATH = ICONS_DIRECTORY / "down_arrow_icon_black.png"
DOWN_ARROW_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "down_arrow_icon_off_white.png"
DOWN_ARROW_ICON_OFF_BLACK_PATH = ICONS_DIRECTORY / "down_arrow_icon_off_black.png"

RIGHT_ARROW_ICON_OFF_BLACK_PATH = ICONS_DIRECTORY / "right_arrow_icon_off_black.png"

LEFT_ARROW_ICON_OFF_BLACK_PATH = ICONS_DIRECTORY / "left_arrow_icon_off_black.png"

DOWN_ARROW_2_ICON_WHITE_PATH = ICONS_DIRECTORY / "down_arrow_2_icon_white.png"
DOWN_ARROW_2_ICON_BRAND_PATH = ICONS_DIRECTORY / "down_arrow_2_icon_brand.png"

RIGHT_ARROW_2_ICON_WHITE_PATH = ICONS_DIRECTORY / "right_arrow_2_icon_white.png"
RIGHT_ARROW_2_ICON_BRAND_PATH = ICONS_DIRECTORY / "right_arrow_2_icon_brand.png"

WARNING_ICON_OFF_WHITE_PATH = ICONS_DIRECTORY / "warning_icon_off_white.png"

BLENDER_ICON_PATH = ICONS_DIRECTORY / "blender_icon.png"
MAYA_ICON_PATH = ICONS_DIRECTORY / "maya_icon.png"
MOE_ICON_PATH = ICONS_DIRECTORY / "moe_icon.png"
VRAY_ICON_PATH = ICONS_DIRECTORY / "vray_icon.png"
EMPTY_ICON_PATH = ICONS_DIRECTORY / "empty_icon.png"

# icons
SPINNER_GIF = str(SPINNER_GIF_PATH)
SPINNER_LARGE_GIF = str(SPINNER_LARGE_GIF_PATH)
LOADING_BAR_GIF = str(LOADING_BAR_GIF_PATH)

# the url of the render manager
REGISTER_ACCOUNT_LINK = "https://portal.gridmarkets.com/init/default/register"
FORGOT_PASSWORD_URL = "https://portal.gridmarkets.com/init/default/lost_password"
RENDER_MANAGER_URL = "https://portal.gridmarkets.com"
COST_CALCULATOR_URL = "https://www.gridmarkets.com/calculator"
FAQ_URL = "https://www.gridmarkets.com/faq"
SECURITY_URL = "https://www.gridmarkets.com/security"
GRIDMARKETS_WEBSITE_URL = "https://www.gridmarkets.com/"
ENVOY_URL = "https://www.gridmarkets.com/envoy"
ENVOY_LOGS_URL = "http://localhost:8090/logs"
SUPPORT_EMAIL = "support@gridmarkets.com"

CURRENT_SCENE_PROJECT_OPTION_INDEX = -1

DEFAULT_MAX_SIZE = 16777215

# These are the max int values for qt which may be different from sys.maxint
# use 32 bit max int values
MAX_INT = math.pow(2, 31) - 1
MIN_INT = -MAX_INT - 1

AVAILABLE_CREDITS_SUFFIX = " Credits"
AVAILABLE_CREDITS_DECIMAL_PLACES = 2
CPS_HEARTBEAT_TIMER_INTERVAL = 10000
RELOAD_SCENE_SETTINGS_SLEEP_INTERVAL = 1

MARGIN_SIZE = 25
MAX_TOP_BANNER_WIDTH = MARGIN_SIZE*2 + 10

class STYLE_PROPS(str, enum.Enum):
    H1 = 'h1'
    H2 = 'h2'
    H3 = 'h3'
    H4 = 'h4'
    H4_5 = 'h4_5'
    H5 = 'h5'
    BRAND = 'brand'
    BORDERED = 'bordered'
    POPUP = 'popup'
    CONSOLE = "console"
    HEADER = 'header'
    MONOSPACE = 'monospace'
    TOOL_TIP = 'tool-tip'
    LEFT_BANNER = 'left-banner'
    BACKGROUND_LEVEL_0 = 'background-level-0'
    BACKGROUND_LEVEL_1 = 'background-level-1'
    BACKGROUND_LEVEL_2 = 'background-level-2'
    MAIN_CONTAINER_WIDGET = 'main-container-widget'
    STATEMENT = 'statement'
    EXPRESSION = 'expression'
    EXPRESSION_TRUE = 'true-expression'
    EXPRESSION_FALSE = 'false-expression'
    EXPRESSION_INTEGER = 'integer-expression'
    RUNTIME_EXPRESSION = 'runtime-expression'
    NOTIFICATION_INFO = 'notification-info'
    APP_INFER_CHECKBOX = 'app-infer-checkbox'
    SMALL_BTN = 'small-btn'
    TAB_BTN = 'tab-btn'
    TAB_BTN_SELECTED = 'tab-btn-selected'


class WhiteTheme(str, enum.Enum):
    WHITE = u'white'
    GREY_01 = u'#f3f3f3'
    BLACK = u'black'
    BRAND_COLOR = u'#38b6ff'
    BRAND_COLOR_HIGHLIGHT = u'#6cc7fc'
    FOREGROUND_COLOR = u'black'
    FOREGROUND_COLOR_2 = u'#333333'
    FOREGROUND_COLOR_3 = u'#4e4e4e'
    BACKGROUND_COLOR_0 = u'white'
    BACKGROUND_COLOR_1 = u'white'
    BACKGROUND_COLOR_2 = u'#f8f8f8'
    MENU_BACKGROUND_COLOR = BACKGROUND_COLOR_1
    ABSTRACT_ITEM_VIEW_BACKGROUND = BACKGROUND_COLOR_1
    ABSTRACT_ITEM_VIEW_ALTERNATE_BACKGROUND = BACKGROUND_COLOR_2
    COLOR_DISABLED = u'#e8e8e8'
    BACKGROUND_COLOR_DISABLED = u'#c4c4c4'
    BORDER_COLOR = u'#c9c9c9'
    BORDER_COLOR_HOVER = u'#494949'
    BORDER_COLOR_DISABLED = COLOR_DISABLED
    LEFT_BANNER_BACKGROUND_COLOR = "#dddddd"
    PLACEHOLDER_TEXT_COLOR = u'#222222'
    CONSOLE_BACKGROUND_COLOR = u'#232323'
    CHECK_BOX_CHECKED_BACKGROUND_COLOR = BLACK
    CHECK_BOX_CHECKED_BORDER_COLOR = BLACK
    CHECK_BOX_CHECKED__HOVER_BACKGROUND_COLOR = u'#313131'
    CHECK_BOX_CHECKED__HOVER_BORDER_COLOR = u'#313131'
    PUSH_BUTTON_COLOR = u'#dddddd'
    PUSH_BUTTON_PRESSED_COLOR = u'#a5a5a5'
    SCROLL_BAR_BACKGROUND = u'#ebebeb'
    SCROLL_BAR_HANDLE_BACKGROUND = u'#c6c6c6'

    TAB_BAR_TAB_BACKGROUND = BACKGROUND_COLOR_2
    TAB_BAR_TAB_BACKGROUND_SELECTED = BACKGROUND_COLOR_2
    TAB_BAR_TAB_BACKGROUND_HOVERED = BACKGROUND_COLOR_1
    TAB_BAR_TAB_BACKGROUND_SELECTED_HOVERED = BACKGROUND_COLOR_2

    NOTIFICATION_INFO_COLOR = '#2282ba'


class BlackTheme(str, enum.Enum):
    WHITE = u'white'
    GREY_01 = u'#f3f3f3'
    BLACK = u'black'
    BRAND_COLOR = u'#38b6ff'
    BRAND_COLOR_HIGHLIGHT = u'#6cc7fc'
    FOREGROUND_COLOR = u'white'
    FOREGROUND_COLOR_2 = u'#efefef'
    FOREGROUND_COLOR_3 = u'#dbdbdb'
    BACKGROUND_COLOR_0 = u'#2d2d2d'
    BACKGROUND_COLOR_1 = u'#222222'
    BACKGROUND_COLOR_2 = u'#171717'
    MENU_BACKGROUND_COLOR = u'#333333'
    ABSTRACT_ITEM_VIEW_BACKGROUND = BACKGROUND_COLOR_1
    ABSTRACT_ITEM_VIEW_ALTERNATE_BACKGROUND = BACKGROUND_COLOR_2
    COLOR_DISABLED = u'#474747'
    BACKGROUND_COLOR_DISABLED = u'#666666'
    BORDER_COLOR = u'#5d5d5d'
    BORDER_COLOR_HOVER = u'#bababa'
    BORDER_COLOR_DISABLED = COLOR_DISABLED
    LEFT_BANNER_BACKGROUND_COLOR = u'black'
    PLACEHOLDER_TEXT_COLOR = u'#ffffff'
    CONSOLE_BACKGROUND_COLOR = BLACK
    CHECK_BOX_CHECKED_BACKGROUND_COLOR = BACKGROUND_COLOR_2
    CHECK_BOX_CHECKED_BORDER_COLOR = BORDER_COLOR
    CHECK_BOX_CHECKED__HOVER_BACKGROUND_COLOR = u'#313131'
    CHECK_BOX_CHECKED__HOVER_BORDER_COLOR = BORDER_COLOR_HOVER
    PUSH_BUTTON_COLOR = u'#565656'
    PUSH_BUTTON_PRESSED_COLOR = u'#868686'
    SCROLL_BAR_BACKGROUND = BACKGROUND_COLOR_1
    SCROLL_BAR_HANDLE_BACKGROUND = u'#717172'

    TAB_BAR_TAB_BACKGROUND = u'#272727'
    TAB_BAR_TAB_BACKGROUND_SELECTED = BACKGROUND_COLOR_0
    TAB_BAR_TAB_BACKGROUND_HOVERED = u'#333333'
    TAB_BAR_TAB_BACKGROUND_SELECTED_HOVERED = TAB_BAR_TAB_BACKGROUND_HOVERED

    NOTIFICATION_INFO_COLOR = '#2282ba'

""" GM Meta Constants """

ROOT_ATTRIBUTE_ID = "GM_PROJECT_NAME"
BLENDER_VERSIONS_ENUM_ID = "BLENDER_VERSION"

EXTERNAL_FILES_DIR_NAME = "gm_external_files"
PROJECT_NAME_DESCRIPTION = "The name of the project folder to upload / submit your job to (As seen in Envoy)."
SUBMISSION_NAME_DESCRIPTION = "The name of submission."
SUBMISSION_COMMENT_DESCRIPTION = "An optional description for the submission."


class JOB_DEFINITION_IDS(str, enum.Enum):
    BLENDER = 'BLENDER'
    MAYA = 'MAYA'
    MAX = 'MAX'
    MOE_DOCKING = 'MOE_DOCKING'
    NUKE = 'NUKE'


class API_KEYS(str, enum.Enum):
    IS_SAVED = "is_saved"
    PROJECT_NAME = "project_name"
    SUBMISSION_NAME = "submission_name"
    SUBMISSION_COMMENT = "submission_comment"
    JOB_NAME = "name"
    APP = "app"
    APP_VERSION = "app_version"
    OPERATION = "operation"
    PLUGINS = "plugins"
    CREDITS_BUDGET = "credits_budget"
    CREDITS_BUDGET_ACTION = "credits_budget_action"
    PATH = "path"
    FRAMES = "frames"
    OUTPUT_PATH = "output_path"
    OUTPUT_PREFIX = "output_prefix"
    OUTPUT_FORMAT = "output_format"
    OUTPUT_MULTIPASS_PREFIX = "output_multipass_prefix"
    OUTPUT_MULTIPASS_FORMAT = "output_multipass_format"
    RENDER_ENGINE = "engine"
    RENDERER = "renderer"
    CAMERA = "camera"
    OUTPUT_WIDTH = "output_width"
    OUTPUT_HEIGHT = "output_height"
    SCENE_FILE_PATH_UNUSED = "scene_file_path_unused"
    GPU = "gpu"
    MACHINE_TYPE = "machine_type"
    INSTANCES = "instances"
    PROJECT_TYPE_ID = "project_type_id"
    PROJ = "proj"
    REMAP = "remap"
    IN_GAMMA = "input_gamma"
    OUT_GAMMA = "output_gamma"
    USE_VRAY_STANDALONE = "use_vray_standalone"
    RENDER_LAYERS = "render_layers"
    TAKE = "take"
    TAKES = "takes"
    AI_SPTX = "ai_sptx"


# The python envoy client treats these attributes as something other than job parameters so we always skip them as they
# are dealt with elsewhere.
ALWAYS_SKIPPED_API_KEYS = [
    API_KEYS.IS_SAVED,
    API_KEYS.SUBMISSION_NAME,
    API_KEYS.SUBMISSION_COMMENT,
    API_KEYS.CREDITS_BUDGET,
    API_KEYS.CREDITS_BUDGET_ACTION
]


class SUBTYPE_KEYS:
    class STRING:
        class PATH(str, enum.Enum):
            FILE_MODE = "FILE_MODE"
            FILE_PATH = "FILE_PATH"
            DIR_PATH = "DIR_PATH"

    class ENUM:
        class PRODUCT_VERSIONS(str, enum.Enum):
            PRODUCT = "PRODUCT"
            MATCH = "MATCH"

        class MACHINE_TYPE(str, enum.Enum):
            PRODUCT = "PRODUCT"
            OPERATION = "OPERATION"
            USE_GPU_DEFAULT = "USE_GPU_DEFAULT"

class JOB_ACTIONS(str, enum.Enum):
    CREDITS_BUDGET_ACTION_ALERT = "CREDITS_BUDGET_ACTION_ALERT"
    CREDITS_BUDGET_ACTION_STOP = "CREDITS_BUDGET_ACTION_STOP"
    CREDITS_BUDGET_ACTION_SUSPEND = "CREDITS_BUDGET_ACTION_SUSPEND"


class PRODUCTS(str, enum.Enum):
    BLENDER = u"blender"
    C4D = u"c4d"
    VRAY = u"vray"
    MOE = u"moe"
    MAYA = u"maya"
    MAX = u"3ds"
    NUKE = u"nuke"


class PLUGINS(str, enum.Enum):
    ARNOLD = "arnold"
    REDSHIFT = "redshift"
    VRAY = "vray"


class OPERATION_TYPES(str, enum.Enum):
    RENDER = u"render"


class BLENDER_VERSIONS(str, enum.Enum):
    V_2_81A = "2.81a"
    V_2_80 = "2.80"
    V_2_79B = "2.79b"
    SERIES_2_8X = "^([2]\\.)([1-7]\\d)"
    SERIES_2_7X = "^([2]\\.)([8]\\d)"


class BLENDER_ENGINES(str, enum.Enum):
    CYCLES = "CYCLES"
    EEVEE = "EEVEE"
    INTERNAL = "INTERNAL"


class VRAY_VERSIONS(str, enum.Enum):
    V_3_60_03 = "3.60.03"
    V_3_60_04 = "3.60.04"
    V_3_60_05 = "3.60.05"
    V_4_02_05 = "4.02.05"
    V_4_10_01 = "4.10.01"
    V_4_10_02 = "4.10.02"


class ATTRIBUTE_NAMES(str, enum.Enum):
    PRODUCT = "app"
