__all__ = 'PluginError'

import sys
import traceback
import typing


class PluginError(Exception):

    def __init__(self, message: typing.Optional['str'] = None):
        Exception.__init__(self, message)

        # Validation to verify that the message is in-fact a string. Without this bugs in the exception handling system
        # itself can go unreported.
        if message is None:
            message = '<empty message>'
        elif not isinstance(message, str):
            message = str(message)

        self._message = message

        try:
            self.traceback = traceback.extract_tb(sys.exc_info()[2])
        except AttributeError:
            self.traceback = None
            pass

    def __str__(self) -> str:
        return self._message

    @property
    def user_message(self) -> str:
        return self._message

    def __repr__(self):

        class_name = self.__class__.__name__
        msg = self._message or '<empty message>'

        return "%s(message=%r)" % (class_name, msg)
