__all__ = 'JobPreset'

from PySide2 import QtCore

from .job_preset_attribute import JobPresetAttribute
from .plugin_accessor import PluginAccessor

import abc
import typing

if typing.TYPE_CHECKING:
    from .job_definition import JobDefinition
    from .submit2gm_plugin import Submit2gmPlugin


class JobPreset(QtCore.QObject, PluginAccessor):

    def __init__(self,
                 plugin: 'Submit2gmPlugin',
                 name: str,
                 identifier: str,
                 job_definition: 'JobDefinition'):

        QtCore.QObject.__init__(self, parent=plugin)
        PluginAccessor.__init__(self, plugin)

        self._name = name
        self._id = identifier
        self._job_definition = job_definition

    def get_name(self) -> str:
        return self._name

    def set_name(self, name: str) -> None:
        self._name = name

    def get_id(self) -> str:
        return self._id

    def get_job_definition(self) -> 'JobDefinition':
        return self._job_definition

    @abc.abstractmethod
    def get_job_preset_attributes(self) -> typing.List['JobPresetAttribute']:
        raise NotImplementedError

    def contains_job_attribute_with_id(self, identifier: str) -> bool:
        for job_preset_attribute in self.get_job_preset_attributes():
            if job_preset_attribute.get_job_attribute().get_id() == identifier:
                return True

        return False

    @abc.abstractmethod
    def get_job_preset_attribute_with_job_attribute_id(self, identifier: str) -> 'JobPresetAttribute':
        raise NotImplementedError

    @abc.abstractmethod
    def get_job_preset_attribute_by_key(self,
                                        key: str,
                                        allow_multiple: bool = False) -> 'JobPresetAttribute':
        raise NotImplementedError

    def contains_job_preset_attribute_with_id(self, identifier: str) -> bool:
        for job_preset_attribute in self.get_job_preset_attributes():
            if job_preset_attribute.get_id() == identifier:
                return True

        return False

    @abc.abstractmethod
    def get_job_preset_attribute_by_id(self, identifier: str) -> 'JobPresetAttribute':
        raise NotImplementedError

    @abc.abstractmethod
    def get_job_preset_attributes_in_group(self, attribute_group: str) -> typing.List['JobPresetAttribute']:
        raise NotImplementedError
