__all__ = ['JobPresetAttribute']

from PySide2 import QtCore

from .plugin_accessor import PluginAccessor

import abc
import typing

if typing.TYPE_CHECKING:
    from .job_attribute import JobAttribute
    from .job_preset import JobPreset
    from .submit2gm_plugin import Submit2gmPlugin


class JobPresetAttribute(QtCore.QObject, PluginAccessor):

    def __init__(self,
                 plugin: 'Submit2gmPlugin',
                 identifier: 'str',
                 job_attribute: 'JobAttribute'):

        QtCore.QObject.__init__(self, parent=plugin)
        PluginAccessor.__init__(self, plugin)
        self._id = identifier
        self._job_attribute = job_attribute

    @abc.abstractmethod
    def get_parent_preset(self) -> 'JobPreset':
        raise NotImplementedError

    def get_job_attribute(self) -> 'JobAttribute':
        return self._job_attribute

    def get_id(self) -> str:
        return self._id

    def get_key(self) -> str:
        return self.get_job_attribute().get_attribute().get_key()

    def get_optional_parent(self) -> 'JobPresetAttribute':
        optional_parent = self.get_job_attribute().get_optional_parent()

        if optional_parent is None:
            raise ValueError("Cannot get optional parent as job preset attribute '' does not have "
                             "one.".format(self.get_id()))

        return self.get_parent_preset().get_job_preset_attribute_with_job_attribute_id(optional_parent)

    @abc.abstractmethod
    def get_value(self) -> any:
        raise NotImplementedError

    @abc.abstractmethod
    def should_skip(self) -> bool:
        raise NotImplementedError
