__all__ = 'ListContainer'

import typing

from .plugin_accessor import PluginAccessor

if typing.TYPE_CHECKING:
    from .submit2gm_plugin import Submit2gmPlugin


class ListContainer(PluginAccessor):

    def __init__(self, plugin: 'Submit2gmPlugin', items: any):
        PluginAccessor.__init__(self, plugin)
        self._items = items
        self._selected = []

    def get_focused_item(self):
        if len(self._selected) == 0:
            return None

        return self._selected[-1]

    def focus_item(self, item, clear_selection=True):
        if not self.contains(item):
            raise ValueError("Can not focus an item not in the list")

        if clear_selection:
            self._selected = []

        self._selected.append(item)

    def get_selected(self):
        return self._selected

    def deselect(self, item):
        if not self.is_selected(item):
            raise ValueError("Can not deselect an item that is not already selected")

        self._selected.remove(item)

    def select_all(self):
        self._selected = []
        for item in self._items:
            self._selected.append(item)

    def deselect_all(self):
        self._selected = []

    def get_all(self):
        return self._items

    def get_at(self, index):

        if index < 0 or index >= len(self._items):
            return None

        return self._items[index]

    def append(self, item, focus_new_item = True):
        self._items.append(item)

        if focus_new_item:
            self.focus_item(item)

    def reset(self):
        self.deselect_all()
        while self._items:
            self.remove(self._items[0])

    def remove(self, item):
        if item in self._selected:
            self._selected.remove(item)

        self._items.remove(item)

    def remove_focused(self):
        if self._selected:
            focused_item = self._selected[-1]
            self.remove(focused_item)

    def remove_selected(self):
        while len(self._selected) > 0:
            item = self._selected[-1]
            self.remove(item)

    def contains(self, item):
        return item in self._items

    def get_index(self, item):
        return self._items.index(item)

    def is_selected(self, item):
        return item in self._selected

    def size(self):
        return len(self._items)
