__all__ = 'LogItem'

from PySide2 import QtCore

import typing

if typing.TYPE_CHECKING:
    import datetime


class LogItem(QtCore.QObject):
    """
    A class representing a single logged message plus all the relevant accompanying information.
    """

    def __init__(self,
                 logger_name: str,
                 level: str,
                 date_time: 'datetime.datetime',
                 message: str):
        QtCore.QObject.__init__(self)
        self._logger_name = logger_name
        self._level = level
        self._date_time = date_time
        self._message = message

    def get_logger_name(self) -> str:
        return self._logger_name

    def get_level(self) -> str:
        return self._level

    def get_date_time(self) -> 'datetime.datetime':
        return self._date_time

    def get_date(self) -> str:
        return self._date_time.strftime("%Y-%m-%d")

    def get_time(self) -> str:
        return self._date_time.strftime("%H:%M:%S")

    def get_message(self) -> str:
        return self._message

    def serialize(self) -> str:
        return self.get_date() + ' ' + \
               self.get_time() + ' ' + \
               self.get_logger_name() + ' ' + \
               self.get_level() + ' ' + \
               self.get_message()
