__all__ = 'Logger'

from PySide2 import QtCore

import datetime
import logging
import typing

from .log_item import LogItem

if typing.TYPE_CHECKING:
    from .logging_coordinator import LoggingCoordinator


class Logger(QtCore.QObject):

    message = QtCore.Signal(LogItem)

    def __init__(self, name: str, logging_coordinator: 'LoggingCoordinator'):
        QtCore.QObject.__init__(self)
        self._logger = logging.getLogger(name)
        self._name = name
        self._logging_coordinator = logging_coordinator

    def _log(self, msg, level) -> None:
        log_item = LogItem(self._name, level, datetime.datetime.now(), msg)

        app = QtCore.QCoreApplication.instance()
        log_item.moveToThread(app.thread())
        log_item.setParent(app)

        self.message.emit(log_item)

    def debug(self, msg, *args, **kwargs) -> None:
        self._log(msg, "DEBUG")
        self._logger.debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs) -> None:
        self._log(msg, "INFO")
        self._logger.info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs) -> None:
        self._log(msg, "WARNING")
        self._logger.warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs) -> None:
        self._log(msg, "ERROR")
        self._logger.error(msg, *args, **kwargs)

    def exception(self, msg, *args, **kwargs) -> None:
        self._log(msg, "ERROR")
        self._logger.exception(msg, *args, exc_info=True, **kwargs)

    def critical(self, msg, *args, **kwargs) -> None:
        self._log(msg, "ERROR")
        self._logger.critical(msg, *args, **kwargs)