__all__ = 'LoggingCoordinator'

from PySide2 import QtCore

import io
import os
import queue
import typing

from .plugin_accessor import PluginAccessor
from .logger import Logger

if typing.TYPE_CHECKING:
    from .submit2gm_plugin import Submit2gmPlugin
    from .widgets.log_history_container_widget import LogHistoryContainerWidget
    from .log_item import LogItem


class LoggingCoordinator(QtCore.QObject, PluginAccessor):
    """
    A class for creating loggers and storing their messages in a list that the ui can draw in a thread safe manner.
    """

    def __init__(self, plugin: 'Submit2gmPlugin', log_history_container: 'LogHistoryContainerWidget'):
        QtCore.QObject.__init__(self)
        PluginAccessor.__init__(self, plugin)
        self._log_history_container = log_history_container

    def get_logger(self, name: typing.Optional[str] = None) -> 'Logger':
        logger = Logger(name, self)
        logger.message.connect(self._log_history_container.append, type=QtCore.Qt.QueuedConnection)
        return logger

    def get_log_history_container(self) -> 'LogHistoryContainerWidget':
        return self._log_history_container

    def serialize_logs(self) -> str:
        sys_info = self.get_plugin().get_plugin_utils().get_sys_info()

        output = ''

        for log_item in self.get_log_history_container().get_all():
            output += log_item.serialize() + os.linesep

        return sys_info + os.linesep + output

    def copy_logs_to_clipboard(self) -> None:
        plugin_utils = self.get_plugin().get_plugin_utils()
        plugin_utils.copy_to_clipboard(self.serialize_logs())

    def save_logs_to_file(self, file_path: str) -> None:
        with io.open(file_path, 'w', encoding='utf-8') as f:
            f.write(self.serialize_logs())
            f.close()