__all__ = 'PackedProject'

import pathlib
import typing

from .local_project import LocalProject
from .utils import remove_directory


class PackedProject(LocalProject):

    def __init__(self,
                 name: str,
                 root_dir: pathlib.Path,
                 files: typing.Set[pathlib.Path],
                 attributes: typing.Dict[str, any],
                 pre_packed_local_root: typing.Optional[pathlib.Path] = None):

        LocalProject.__init__(self,
                              name,
                              root_dir,
                              files,
                              attributes)

        self._pre_packed_local_root = pre_packed_local_root

    def get_relative_file_path(self, file: pathlib.Path) -> pathlib.Path:
        root_dir = self.get_root_dir()

        if not file.is_absolute():
            raise ValueError("file \"" + str(file) + "\" must be absolute")

        # If the file is external to to the projects root directory then it cannot be made relative
        if file.anchor != root_dir.anchor:
            return file

        return file.relative_to(root_dir)

    def get_relative_files(self) -> typing.Set[pathlib.Path]:
        relative_files = map(self.get_relative_file_path, self.get_files())
        return set(relative_files)

    def get_pre_packed_local_root(self) -> pathlib.Path:
        if self._pre_packed_local_root is not None:
            return self._pre_packed_local_root

        return self.get_root_dir()

    def set_pre_packed_local_root(self, pre_packed_local_root: pathlib.Path) -> None:
        self._pre_packed_local_root = pre_packed_local_root

    def exists(self) -> bool:
        if self.get_root_dir().is_dir():
            return True

    def delete(self) -> None:
        if self.exists():
            remove_directory(self.get_root_dir())
