__all__ = ['PackedSceneBuilder']

from abc import abstractmethod
import typing

from .plugin_accessor import PluginAccessor

if typing.TYPE_CHECKING:
    from .submit2gm_plugin import Submit2gmPlugin
    from .dynamic_job_preset import DynamicJobPreset
    from submit2gm.packed_project import PackedProject
    from submit2gm.widgets.node_editor.node_widget import DependencyNode


class PackedSceneBuilder(PluginAccessor):

    def __init__(self, plugin: 'Submit2gmPlugin'):
        PluginAccessor.__init__(self, plugin)

    @abstractmethod
    def _build_scene_internal(self,
                              project_name: str,
                              save_project_if_required: bool,
                              attributes: typing.Dict[str, any],
                              dynamic_job_presets: typing.List['DynamicJobPreset'],
                              dependency_graph: typing.Dict[str, 'DependencyNode']) -> 'PackedProject':
        raise NotImplementedError

    def get_current_scene(self,
                          project_name: str,
                          save_project_if_required: bool,
                          attributes: typing.Dict[str, any],
                          dynamic_job_presets: typing.List['DynamicJobPreset'],
                          dependency_graph: typing.Dict[str, 'DependencyNode']) -> 'PackedProject':
        """
        todo make it so you only pass the project attribute and all other values are passed from it. Requires project
          attribute instances
        """

        if attributes is None:
            attributes = {}

        packed_project = self._build_scene_internal(project_name,
                                                    save_project_if_required,
                                                    attributes,
                                                    dynamic_job_presets=dynamic_job_presets,
                                                    dependency_graph=dependency_graph)

        packed_project.set_name(project_name)

        if attributes:
            for key, value in attributes.items():

                # If the key is not already in the packed project's attribute list
                if key not in packed_project.get_attributes():
                    # Then set the attribute
                    packed_project.set_attribute(key, value)

        return packed_project

    @abstractmethod
    def get_product_type(self) -> str:
        """ Returns the project type for the current scene. """
        raise NotImplementedError

    @abstractmethod
    def get_product_version(self) -> str:
        """ Returns the project version for the current scene. """
        raise NotImplementedError
