__all__ = 'PluginUtils'

import PySide2
from PySide2 import QtGui

from . import constants
from .plugin_accessor import PluginAccessor


class PluginUtils(PluginAccessor):

    def get_application_version(self) -> str:
        return ""

    def get_sys_info(self) -> str:
        import platform
        import os

        plugin = self.get_plugin()
        plugin_version = plugin.get_version()

        sys_info = "Operating System: " + platform.platform() + os.linesep
        sys_info += constants.PROGRAM_NAME + " Version: " + plugin_version.to_string()

        try:
            cps = plugin.get_active_command_port_server()
            sys_info += os.linesep
            sys_info += os.linesep
            sys_info += "Active Command Port Server:" + os.linesep
            sys_info += "Address: " + cps.get_address() + os.linesep
            sys_info += "Product: " + cps.get_product() + os.linesep
            sys_info += "PySide2 version: " + PySide2.__version__ + os.linesep
            if cps.get_meta_info():
                sys_info += cps.get_meta_info() + os.linesep
        except ValueError as e:
            pass

        return sys_info

    def copy_to_clipboard(self, value: str) -> None:
        clipboard = QtGui.QGuiApplication.clipboard()
        clipboard.setText(value)
