__all__ = 'Project'

from abc import abstractmethod
import pathlib
import typing

if typing.TYPE_CHECKING:
    from .watch_pattern import WatchPattern


class Project:

    def __init__(self,
                 name: str,
                 root_dir: pathlib.Path,
                 files: typing.Set[pathlib.Path],
                 attributes: typing.Dict):
        self._name = name
        self._root_dir = root_dir
        self._files = files
        self._attributes = attributes
        self._watch_patterns: typing.List['WatchPattern'] = []

    def get_name(self) -> str:
        return self._name

    def set_name(self, name: str) -> None:
        self._name = name

    def get_attributes(self) -> typing.Dict:
        return self._attributes

    def get_attribute(self, key: str) -> any:
        return self._attributes[key]

    def set_attribute(self, key: str, value: any) -> None:
        self._attributes[key] = value

    @abstractmethod
    def exists(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    def delete(self) -> None:
        raise NotImplementedError

    def get_root_dir(self) -> pathlib.Path:
        return self._root_dir

    def get_files(self) -> typing.Set[pathlib.Path]:
        return self._files

    def add_files(self, files: typing.Set[pathlib.Path]) -> None:
        self._files = self._files | files

    def set_files(self, files: typing.Set[pathlib.Path]) -> None:
        self._files = files

    def get_watch_patterns(self) -> typing.List['WatchPattern']:
        return self._watch_patterns

    def set_watch_patterns(self, watch_patterns: typing.List['WatchPattern']) -> None:
        self._watch_patterns = watch_patterns

    @abstractmethod
    def get_size(self) -> int:
        """
        Gets the size of the project in bytes.
        """
        raise NotImplementedError
