from . import blender_constants

import typing

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import CommandPortServer


def is_saved(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
is_saved = bpy.data.is_saved
""", return_variable_name='is_saved')


def is_dirty(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
is_dirty = bpy.data.is_dirty
""", return_variable_name='is_dirty')


def save(cps: 'CommandPortServer'):
    return cps.eval("""
bpy.ops.wm.save_as_mainfile(check_existing=False)
""")


def get_file_path(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
blend_file_path = bpy.context.blend_data.filepath.replace('\\\\', '/')
""", return_variable_name='blend_file_path')


def get_scene_name(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
blend_file_name = bpy.path.basename(bpy.context.blend_data.filepath) 
""", return_variable_name='blend_file_name')


def get_active_camera_name(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
active_camera = bpy.context.scene.camera
active_camera_name = ''

if active_camera is not None and active_camera.type == 'CAMERA':
    active_camera_name = active_camera.name
""", return_variable_name='active_camera_name')


def get_version_tuple(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
blender_version = bpy.app.version
""", return_variable_name='blender_version')


def get_render_engine(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
render_engine =  bpy.context.engine
""", return_variable_name='render_engine')


def get_output_path(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
import os
output_path = bpy.path.abspath(bpy.context.scene.render.filepath)
if os.name == 'nt': output_path = output_path.replace('\\\\', '/')
""", return_variable_name='output_path')


def get_output_format(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
output_format = bpy.context.scene.render.image_settings.file_format
""", return_variable_name='output_format')


def get_frame_range_str(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
frame_range_str = str(bpy.context.scene.frame_start) + ' ' + str(bpy.context.scene.frame_end) + ' ' + str(bpy.context.scene.frame_step)
""", return_variable_name='frame_range_str')


def get_local_download_path(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
local_download_path = ""
import pathlib
local_download_path = pathlib.Path(bpy.path.abspath(bpy.context.scene.render.filepath)).resolve().as_posix()
""", return_variable_name='local_download_path')


def save_main_file(cps: 'CommandPortServer', target: str):
    return cps.eval("""
bpy.ops.wm.save_as_mainfile(copy=True, filepath=fr'{}', relative_remap=True, compress=True)
""".format(target))


def trace_assets(cps: 'CommandPortServer', target_file: str) -> typing.List[str]:
    return cps.eval_cmd("""
import pathlib
from blender_asset_tracer import blendfile, trace
blend_file_path = pathlib.Path(bpy.context.blend_data.filepath.replace('\\\\', '/'))
project_files = set([blend_file_path.as_posix()])

try:
    for dep in trace.deps(blend_file_path):
        
        actual_type = dep.block.dna_type.dna_type_id.decode()

        for file in dep.files():
            project_files.add(file.as_posix())
finally:
    blendfile.close_all_cached()

bpy.ops.wm.save_as_mainfile(check_existing=False)

project_files = list(project_files)
""".format(target_file), return_variable_name='project_files')
