__all__ = 'BlenderJobPresetValidator'

import typing

from submit2gm import constants
from submit2gm.job_preset_validator import JobPresetValidator
from . import blender_constants

if typing.TYPE_CHECKING:
    from submit2gm.dynamic_job_preset import DynamicJobPreset


class BlenderJobPresetValidator(JobPresetValidator):

    def validate(self, job_preset: 'DynamicJobPreset') -> typing.Tuple[bool, str]:
        is_valid = True
        validation_message = ""

        # Validate Blender renderer
        renderer_attribute = job_preset.get_job_preset_attribute_by_key(constants.API_KEYS.RENDER_ENGINE)
        renderer = renderer_attribute.eval()
        if renderer not in blender_constants.SUPPORTED_RENDER_ENGINES:
            renderer_name = renderer

            if renderer == blender_constants.RENDER_ENGINE_CYCLES:
                renderer_name = "Cycles"
            elif renderer == blender_constants.RENDER_ENGINE_EEVEE:
                renderer_name = "Eevee"
            elif renderer == blender_constants.RENDER_ENGINE_BLENDER_INTERNAL:
                renderer_name = "Blender Internal"
            elif renderer == blender_constants.RENDER_ENGINE_BLENDER_WORKBENCH:
                renderer_name = "Blender Workbench"
            elif renderer == blender_constants.RENDER_ENGINE_VRAY_RT:
                renderer_name = "V-Ray"

            return False, "Render engine '%s' is not currently supported." % renderer_name

        return is_valid, validation_message
