__all__ = 'MoeAttributeSource'

from submit2gm import constants
from submit2gm.application_attribute_source import ApplicationAttributeSource
from submit2gm.errors.application_attribute_not_found import ApplicationAttributeNotFound
from submit2gm.attribute_types import EnumItem


class MoeAttributeSource(ApplicationAttributeSource):

    def get_enum_items(self, app: str, app_version: str, key: str):
        plugin = self.get_plugin()

        if key == constants.API_KEYS.APP_VERSION:
            items = []
            product_versions = self.get_plugin().get_api_client().get_product_versions(app)
            for product_version in product_versions:
                items.append(EnumItem(product_version, product_version, u""))
            return items, self._get_enum_index(app, app_version, key, items)

        if key == constants.API_KEYS.MACHINE_TYPE:
            return plugin.get_api_client().get_machines_options(constants.PRODUCTS.MOE,
                                                                constants.OPERATION_TYPES.RENDER,
                                                                use_gpu_default=True)

        raise ApplicationAttributeNotFound(app, app_version, key)

    def get_attribute_value(self, app: str, app_version: str, key: str):

        if app != constants.PRODUCTS.MOE:
            raise ValueError(str(app))

        if key == constants.API_KEYS.IS_SAVED:
            return True

        if key == constants.API_KEYS.PROJECT_NAME:
            return 'Project Name'

        if key == constants.API_KEYS.SUBMISSION_NAME:
            return "Submission Name"

        if key == constants.API_KEYS.APP:
            return constants.PRODUCTS.MOE

        if key == constants.API_KEYS.APP_VERSION:
            moe_product_versions = self.get_plugin().get_api_client().get_product_versions(constants.PRODUCTS.MOE)
            return moe_product_versions[0]

        raise ApplicationAttributeNotFound(app, app_version, key)
