__all__ = 'NukeAttributeSource'

import pathlib
import typing

from submit2gm import constants
from submit2gm.application_attribute_source import ApplicationAttributeSource
from submit2gm.errors.application_attribute_not_found import ApplicationAttributeNotFound
from submit2gm.attribute_types import EnumItem
from . import nuke_cpc


class NukeAttributeSource(ApplicationAttributeSource):

    def get_enum_items(self, app: str, version: str, key: str):
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()

        if key == constants.API_KEYS.APP_VERSION:
            items = []

            current_nuke_version = nuke_cpc.get_version_string(cps)
            items.append(EnumItem(current_nuke_version, current_nuke_version, u""))

            return items, 0

        if key == constants.API_KEYS.MACHINE_TYPE:
            items = list()

            api_client = self.get_plugin().get_api_client()
            machines = api_client.get_machines(constants.PRODUCTS.NUKE, constants.OPERATION_TYPES.RENDER)

            index = 0
            default = 0
            for machine in machines:
                if not machine.is_gpu_machine():
                    items.append(EnumItem(machine.get_id(), machine.get_name(), u""))

                    if machine.is_default():
                        default = index

                    index = index + 1

            return items, default

        raise ApplicationAttributeNotFound(app, version, key)

    def get_attribute_value(self, app: str, app_version: str, key: str):
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()

        if key == constants.API_KEYS.IS_SAVED:
            file_name = nuke_cpc.get_script_name(cps)
            return len(file_name) > 0

        if key == constants.API_KEYS.PROJECT_NAME:
            return pathlib.Path(nuke_cpc.get_project_dir(cps)).stem

        if key == constants.API_KEYS.SUBMISSION_NAME:
            file_name = pathlib.Path(nuke_cpc.get_script_name(cps)).name

            if not file_name:
                return "Untitled"

            return file_name

        if key == constants.API_KEYS.JOB_NAME:
            return 'Nuke job'

        if key == constants.API_KEYS.APP:
            return constants.PRODUCTS.NUKE

        if key == constants.API_KEYS.APP_VERSION:
            return nuke_cpc.get_version_string(cps)

        if key == constants.API_KEYS.SCENE_FILE_PATH_UNUSED:
            file_path = nuke_cpc.get_script_name(cps)
            return file_path

        if key == constants.API_KEYS.FRAMES:
            return nuke_cpc.get_frame_range_str(cps)

        raise ApplicationAttributeNotFound(app, app_version, key)
