__all__ = 'Submit2GMPackedSceneBuilder'

import typing

from . import constants
from .packed_scene_builder import PackedSceneBuilder

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin
    from submit2gm.packed_project import PackedProject
    from submit2gm.dynamic_job_preset import DynamicJobPreset
    from submit2gm.widgets.node_editor.node_widget import DependencyNode
    from submit2gm.submit2gm_dyn.blender.blender_packed_scene_builder import BlenderPackedSceneBuilder
    from submit2gm.submit2gm_dyn.c4d.c4d_packed_scene_builder import C4DPackedSceneBuilder
    from submit2gm.submit2gm_dyn.max.max_packed_scene_builder import MaxPackedSceneBuilder
    from submit2gm.submit2gm_dyn.maya.maya_packed_scene_builder import MayaPackedSceneBuilder
    from submit2gm.submit2gm_dyn.nuke.nuke_packed_scene_builder import NukePackedSceneBuilder


class Submit2GMPackedSceneBuilder(PackedSceneBuilder):

    def __init__(self, plugin: 'Submit2gmPlugin'):
        PackedSceneBuilder.__init__(self, plugin)
        self._blender_packed_scene_builder: typing.Optional['BlenderPackedSceneBuilder'] = None
        self._c4d_packed_scene_builder: typing.Optional['C4DPackedSceneBuilder'] = None
        self._max_packed_scene_builder: typing.Optional['MaxPackedSceneBuilder'] = None
        self._maya_packed_scene_builder: typing.Optional['MayaPackedSceneBuilder'] = None
        self._nuke_packed_scene_builder: typing.Optional['NukePackedSceneBuilder'] = None

    def get_blender_packed_scene_builder(self) -> 'BlenderPackedSceneBuilder':
        if self._blender_packed_scene_builder is None:
            from submit2gm.submit2gm_dyn.blender.blender_packed_scene_builder import BlenderPackedSceneBuilder
            self._blender_packed_scene_builder = BlenderPackedSceneBuilder(self.get_plugin())
        return self._blender_packed_scene_builder

    def get_c4d_packed_scene_builder(self) -> 'C4DPackedSceneBuilder':
        if self._c4d_packed_scene_builder is None:
            from submit2gm.submit2gm_dyn.c4d.c4d_packed_scene_builder import C4DPackedSceneBuilder
            self._c4d_packed_scene_builder = C4DPackedSceneBuilder(self.get_plugin())
        return self._c4d_packed_scene_builder

    def get_max_packed_scene_builder(self) -> 'MaxPackedSceneBuilder':
        if self._max_packed_scene_builder is None:
            from submit2gm.submit2gm_dyn.max.max_packed_scene_builder import MaxPackedSceneBuilder
            self._max_packed_scene_builder = MaxPackedSceneBuilder(self.get_plugin())
        return self._max_packed_scene_builder

    def get_maya_packed_scene_builder(self) -> 'MayaPackedSceneBuilder':
        if self._maya_packed_scene_builder is None:
            from submit2gm.submit2gm_dyn.maya.maya_packed_scene_builder import MayaPackedSceneBuilder
            self._maya_packed_scene_builder = MayaPackedSceneBuilder(self.get_plugin())
        return self._maya_packed_scene_builder

    def get_nuke_packed_scene_builder(self) -> 'NukePackedSceneBuilder':
        if self._nuke_packed_scene_builder is None:
            from submit2gm.submit2gm_dyn.nuke.nuke_packed_scene_builder import NukePackedSceneBuilder
            self._nuke_packed_scene_builder = NukePackedSceneBuilder(self.get_plugin())
        return self._nuke_packed_scene_builder

    def _get_packed_scene_builder(self) -> 'PackedSceneBuilder':
        cps = self.get_plugin().get_active_command_port_server()
        product = cps.get_product()

        if product == constants.PRODUCTS.BLENDER:
            return self.get_blender_packed_scene_builder()

        if product == constants.PRODUCTS.C4D:
            return self.get_c4d_packed_scene_builder()

        if product == constants.PRODUCTS.MAX:
            return self.get_max_packed_scene_builder()

        if product == constants.PRODUCTS.MAYA:
            return self.get_maya_packed_scene_builder()

        if product == constants.PRODUCTS.NUKE:
            return self.get_nuke_packed_scene_builder()

        raise ValueError("{} does not support product '{}'".format(constants.PROGRAM_NAME, product))

    def _build_scene_internal(self,
                              project_name: str,
                              save_project_if_required: bool,
                              attributes: typing.Dict[str, any],
                              dynamic_job_presets: typing.List['DynamicJobPreset'],
                              dependency_graph: typing.Dict[str, 'DependencyNode']) -> 'PackedProject':

        return self._get_packed_scene_builder()._build_scene_internal(project_name,
                                                                      save_project_if_required,
                                                                      attributes,
                                                                      dynamic_job_presets,
                                                                      dependency_graph)

    def get_product_type(self) -> str:
        return self._get_packed_scene_builder().get_product_type()

    def get_product_version(self) -> str:
        return self._get_packed_scene_builder().get_product_version()
