__all__ = 'Submit2GMUserInterface'

from PySide2 import QtCore
from PySide2 import QtGui
import typing

from submit2gm import constants, ui_constants
from submit2gm.plugin_accessor import PluginAccessor
from submit2gm.worker_thread import WorkerThread

if typing.TYPE_CHECKING:
    from .submit2gm_plugin import Submit2gmPlugin


class Submit2GMUserInterface(PluginAccessor, QtCore.QObject):
    operation_started = QtCore.Signal()
    operation_finished = QtCore.Signal()

    CUSTOM_FONT_FILES = [
        u"Montserrat/Montserrat-Black.ttf",
        u"Montserrat/Montserrat-BlackItalic.ttf",
        u"Montserrat/Montserrat-Bold.ttf",
        u"Montserrat/Montserrat-BoldItalic.ttf",
        u"Montserrat/Montserrat-ExtraBold.ttf",
        u"Montserrat/Montserrat-ExtraBoldItalic.ttf",
        u"Montserrat/Montserrat-ExtraLight.ttf",
        u"Montserrat/Montserrat-ExtraLightItalic.ttf",
        u"Montserrat/Montserrat-Italic.ttf",
        u"Montserrat/Montserrat-Light.ttf",
        u"Montserrat/Montserrat-LightItalic.ttf",
        u"Montserrat/Montserrat-Medium.ttf",
        u"Montserrat/Montserrat-MediumItalic.ttf",
        u"Montserrat/Montserrat-Regular.ttf",
        u"Montserrat/Montserrat-SemiBold.ttf",
        u"Montserrat/Montserrat-SemiBoldItalic.ttf",
        u"Montserrat/Montserrat-Thin.ttf",
        u"Montserrat/Montserrat-ThinItalic.ttf",
        u"Open_Sans/OpenSans-Bold.ttf",
        u"Open_Sans/OpenSans-BoldItalic.ttf",
        u"Open_Sans/OpenSans-ExtraBold.ttf",
        u"Open_Sans/OpenSans-ExtraBoldItalic.ttf",
        u"Open_Sans/OpenSans-Italic.ttf",
        u"Open_Sans/OpenSans-Light.ttf",
        u"Open_Sans/OpenSans-LightItalic.ttf",
        u"Open_Sans/OpenSans-Regular.ttf",
        u"Open_Sans/OpenSans-SemiBold.ttf",
        u"Open_Sans/OpenSans-SemiBoldItalic.ttf"
    ]

    LOADED_FONTS = []

    def __init__(self, plugin: 'Submit2gmPlugin'):
        PluginAccessor.__init__(self, plugin)
        QtCore.QObject.__init__(self, None)

        self._auth_email_is_valid = True
        self._auth_email_validity_message = u""

        self._auth_access_key_is_valid = True
        self._auth_access_key_validity_message = u""

        self._auth_user_is_valid = True
        self._auth_user_validity_message = u""

        self._is_running_operation = False
        self._running_operation_message = ''

        self._use_dark_theme = True
        self._styled_widgets = []

    def get_auth_email(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_email_input.get_widget().text()

    def set_auth_email(self, email):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_email_input.get_widget().setText(email)

    def get_auth_email_validity_flag(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_email_input.is_valid()

    def set_auth_email_validity_flag(self, is_valid):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_email_input.set_validity_flag(is_valid)

    def get_auth_email_validity_message(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_email_input.get_warning_message()

    def set_auth_email_validity_message(self, message):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_email_input.set_warning_message(message)

    def get_auth_access_key(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_access_key_input.get_widget().text()

    def set_auth_access_key(self, email):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_access_key_input.get_widget().setText(email)

    def get_auth_access_key_validity_flag(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_access_key_input.is_valid()

    def set_auth_access_key_validity_flag(self, is_valid):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_access_key_input.set_validity_flag(is_valid)

    def get_auth_access_key_validity_message(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.auth_access_key_input.get_warning_message()

    def set_auth_access_key_validity_message(self, message):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.auth_access_key_input.set_warning_message(message)

    def get_user_validity_flag(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.sign_in_btn.is_valid()

    def set_user_validity_flag(self, is_valid):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.sign_in_btn.set_validity_flag(is_valid)
        sign_in_widget.sign_in_envoy_btn.set_validity_flag(is_valid)

    def get_user_validity_message(self):
        sign_in_widget = self._main_window.get_sign_in_widget()
        return sign_in_widget.sign_in_btn.get_warning_message()

    def set_user_validity_message(self, message):
        sign_in_widget = self._main_window.get_sign_in_widget()
        sign_in_widget.sign_in_btn.set_warning_message(message)
        sign_in_widget.sign_in_envoy_btn.set_warning_message(message)


    def get_icon(self) -> 'QtGui.QPixmap':
        return ui_constants.GM_ICON_BLACK_AND_WHITE

    def get_left_banner_image(self) -> 'QtGui.QPixmap':
        from PySide2 import QtGui
        return QtGui.QPixmap(str(""))

    def get_left_banner_link(self) -> str:
        return constants.BANNER_URL

    def get_help_url(self) -> str:
        return constants.HELP_URL

    def get_main_window(self):
        return self._main_window

    def get_main_switcher_ui(self):
        return self._main_window.get_main_ui_widget().get_main_ui_switcher()

    def set_main_window(self, main_window):
        self._main_window = main_window

    def is_running_operation(self):
        return self._is_running_operation

    def set_is_running_operation_flag(self, value):
        self._is_running_operation = value

    def get_running_operation_message(self):
        return self._running_operation_message

    def set_running_operation_message(self, message):
        self._running_operation_message = message

    def load_profile(self, user_profile):
        raise NotImplementedError

    def show_log_dates(self):
        raise NotImplementedError

    def set_show_log_dates(self, enabled):
        raise NotImplementedError

    def show_log_times(self):
        raise NotImplementedError

    def set_show_log_times(self, enabled):
        raise NotImplementedError

    def show_logger_names(self):
        raise NotImplementedError

    def set_show_logger_names(self, enabled):
        raise NotImplementedError

    def get_show_hidden_job_preset_attributes(self):
        raise NotImplementedError

    def set_show_hidden_job_preset_attributes(self, value):
        raise NotImplementedError

    def set_running_operation(self, message):
        self.set_is_running_operation_flag(True)
        self.set_running_operation_message(message)
        self.operation_started.emit()

    def hide_running_operation_message(self):
        self.set_is_running_operation_flag(False)
        self.operation_finished.emit()

    def get_worker_thread(self, status_message):

        self._worker_thread = WorkerThread(self.get_plugin())
        self._worker_thread.started.connect(lambda: self.set_running_operation(status_message))
        self._worker_thread.finished.connect(self.hide_running_operation_message)

        return self._worker_thread

    def refresh_remote_root_dirs(self, slot):
        api_client = self.get_plugin().get_api_client()
        worker_thread = self.get_worker_thread(u"Fetching Existing Projects...")
        ################################################################################################################

        target = api_client.get_root_directories

        args = set()

        kwargs = {
            'ignore_cache': True
        }

        worker_thread.process(target, args, kwargs, slot)

    def use_dark_theme(self):
        return self._use_dark_theme

    def toggle_theme(self):
        self._use_dark_theme = not self._use_dark_theme

        for styled_widget in self._styled_widgets:
            self.set_style_sheet(styled_widget)

    def get_theme(self):
        if self.use_dark_theme():
            return constants.BlackTheme
        else:
            return constants.WhiteTheme

    def set_style_sheet(self, widget, theme=None):

        if widget not in self._styled_widgets:
            self._styled_widgets.append(widget)

        widget.setStyleSheet(self.get_style_sheet(theme=theme))

    def get_style_sheet(self, theme=None):

        if theme is None:
            theme = self.get_theme()

        return u"""
        * {
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            font-style: normal;
            color: """ + theme.FOREGROUND_COLOR + u""";
        }

        QWidget {
            background: transparent;
        }

        QWidget:focus {
            outline: 0;
        }

        QWidget[main-container-widget=true] {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
        }

        QWidget[background-off-white=true] {
            background: """ + theme.BACKGROUND_COLOR_2 + u""";
        }

        QWidget[background-level-0=true] {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
        }

        QWidget[background-level-1=true] {
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }

        QWidget[background-level-2=true] {
            background-color: """ + theme.BACKGROUND_COLOR_2 + u""";
        }

        QAbstractScrollArea {
            background: transparent;
        }

        QDialog {
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }

        QGroupBox {
            border: 0px solid transparent;
            border-radius: 0px;
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
            padding: 15px;
        }

        QGroupBox::title {
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            color: """ + theme.FOREGROUND_COLOR + u""";
        }

        QGroupBox[""" + constants.STYLE_PROPS.NOTIFICATION_INFO + """=true] {
            padding: 3px 5px;
            font-family: "Open Sans SemiBold";
            font-size: 14px;
            background-color: """ + theme.NOTIFICATION_INFO_COLOR + u""";
            border-radius: 0px;
        }

        QGroupBox[""" + constants.STYLE_PROPS.STATEMENT + """=true] {
            padding: 5px;
            background-color: 'transparent';
            border-left-width: 2px;
            border-left-style: solid;
            border-left-color: rgba(255, 255, 255, 30%);
            border-radius: 0px;
        }

        QGroupBox[""" + constants.STYLE_PROPS.STATEMENT + """=true][hover=true] {
            background-color: rgba(255, 255, 255, 5%);
            border-left-color: rgba(255, 255, 255, 60%);
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true] {
            padding: 3px 5px;
            background-color: 'transparent';
            border-radius: 2px;
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][hover=true] {
            background-color: rgba(255, 255, 255, 20%);
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][""" + constants.STYLE_PROPS.EXPRESSION_TRUE + """=true] {
            background-color: '#00904b';
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][hover=true][""" + constants.STYLE_PROPS.EXPRESSION_TRUE + """=true] {
            background-color: '#00ad5a';
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][""" + constants.STYLE_PROPS.EXPRESSION_FALSE + """=true] {
            background-color: '#ff000c';
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][hover=true][""" + constants.STYLE_PROPS.EXPRESSION_FALSE + """=true] {
            background-color: '#ff3841';
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][""" + constants.STYLE_PROPS.EXPRESSION_INTEGER + """=true] {
            background-color: '#079093';
            padding-right: 32px;
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][hover=true][""" + constants.STYLE_PROPS.EXPRESSION_INTEGER + """=true] {
            background-color: '#00c3c8';
        }   

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][""" + constants.STYLE_PROPS.RUNTIME_EXPRESSION + """=true] {
            background-color: '#007fff';
        }

        QGroupBox[""" + constants.STYLE_PROPS.EXPRESSION + """=true][hover=true][""" + constants.STYLE_PROPS.RUNTIME_EXPRESSION + """=true] {
            background-color: '#1f8fff';
        }

        QPushButton {
            font-family: "Montserrat SemiBold";
            background: """ + theme.FOREGROUND_COLOR + u""";
            color: """ + theme.PUSH_BUTTON_COLOR + u""";
            padding: 11px 16px;
            border-radius: 4px;
        }

        QPushButton[large-btn=true] {
            margin-top: 6px;
            min-height: 46px;
        }

        QPushButton[small-btn=true] {
            padding: 5px 8px;
        }

        QPushButton:hover {
            color: """ + theme.BACKGROUND_COLOR_1 + u""";
        }

        QPushButton:pressed {
            color: """ + theme.PUSH_BUTTON_PRESSED_COLOR + u""";
        }

        QPushButton:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QPushButton[brand=true] {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.WHITE + u""";
        }

        QPushButton[brand=true]:hover {
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            color: """ + theme.WHITE + u""";
        }

        QPushButton[brand=true]:pressed {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.GREY_01 + u""";
        }

        QPushButton[brand=true]:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QPushButton:default {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.WHITE + u""";
        }

        QPushButton:default:hover {
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            color: """ + theme.WHITE + u""";
        }

        QPushButton:default:pressed {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.GREY_01 + u""";
        }

        QPushButton:default:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }
        
        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN + """=true] {
            font-family: "Montserrat";
            border-radius: 0px;
            padding: 3px 6px 3px 6px;
            color: """ + theme.FOREGROUND_COLOR_2 + u""";
            background: """ + theme.TAB_BAR_TAB_BACKGROUND + u""";
        }
        
        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN + """=true]::menu-indicator { 
            image: none;
            width:0px;
        }
        
        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN + """=true]:hover {
            background: """ + theme.TAB_BAR_TAB_BACKGROUND_HOVERED + u""";
        }

        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN + """=true]:pressed {
            background: """ + theme.TAB_BAR_TAB_BACKGROUND_HOVERED + u""";
        }
        
        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN_SELECTED + """=true] {
            color: """ + theme.FOREGROUND_COLOR_2 + u""";
            background: """ + theme.TAB_BAR_TAB_BACKGROUND_SELECTED + u""";
        }
        
        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN_SELECTED + """=true]:hover {
            background: """ + theme.TAB_BAR_TAB_BACKGROUND_SELECTED_HOVERED + u""";
        }

        QPushButton[""" + constants.STYLE_PROPS.TAB_BTN_SELECTED + """=true]:pressed {
            background: """ + theme.TAB_BAR_TAB_BACKGROUND_SELECTED_HOVERED + u""";
        }

        QPushButton:flat {
            font-family: "Montserrat Medium";
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            padding: 5px 4px;
        }

        QPushButton:flat:hover {
            background: transparent;
            color: """ + theme.BRAND_COLOR + u""";
        }

        QPushButton:flat:pressed  {
            background: transparent;
            color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QPushButton:flat:disabled  {
            background: transparent;
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QPushButton:flat[""" + constants.STYLE_PROPS.BORDERED + """=true] {
            border-width: 1px;
            border-style: solid;
            border-radius: 4px;
            border-color: """ + theme.BORDER_COLOR + """
        }

        QPushButton:flat[""" + constants.STYLE_PROPS.BORDERED + """=true]:hover {
            color: """ + theme.FOREGROUND_COLOR + u""";
            border-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QPushButton:flat[""" + constants.STYLE_PROPS.BORDERED + """=true]:pressed  {
            color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            border-color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QPushButton:flat[""" + constants.STYLE_PROPS.BORDERED + """=true]:disabled  {
            color: """ + theme.COLOR_DISABLED + u""";
            border-color: """ + theme.COLOR_DISABLED + u""";
        }

        QPushButton::menu-indicator {
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""") 0 0 0 0 stretch stretch;
            subcontrol-origin: padding;
            subcontrol-position: right center;
        }

        QPushButton::menu-indicator[noMenuIndicator=true] {
            image: none;
        }


        QToolButton::menu-arrow {
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""") 0 0 0 0 stretch stretch;
            subcontrol-origin: padding;
            subcontrol-position: center center;
        }

        QToolButton[brand=true] {
            font-family: "Montserrat SemiBold";
            padding: 11px 16px;
            padding-right: 20px; /* make way for the popup button */
            border-radius: 4px;
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.WHITE + u""";
        }

        QToolButton[brand=true]:hover {
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            color: """ + theme.WHITE + u""";
        }

        QToolButton[brand=true]:pressed {
            background: """ + theme.BRAND_COLOR + u""";
            color: """ + theme.GREY_01 + u""";
        }

        QToolButton[brand=true]:disabled {
            background: """ + theme.BACKGROUND_COLOR_DISABLED + u""";
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QToolButton::menu-button[brand=true] {
            border: 0px solid transparent;
            width: 20px;
        }

        QToolButton::menu-button:hover[brand=true] {
            border: 0px solid transparent;
        }


        QFrame {
            color: """ + theme.FOREGROUND_COLOR_2 + u""";
            font-family: "Montserrat";
        }

        QFrame[hr=true] {
            min-height: 1px;
            margin: 5px 0px;
            background: """ + theme.BORDER_COLOR + u""";
        }

        QLabel {
            color: """ + theme.FOREGROUND_COLOR_2 + u""";
            font-family: "Montserrat";
        }

        QLabel:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
        }

        QWidget[h1=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 36px;
        }

        QWidget[h2=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 26px;
        }

        QWidget[h3=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 20px;
        }

        QWidget[h4=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 16px;
        }

        QWidget[h4_5=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 14px;
        }

        QWidget[h5=true] {
            color: """ + theme.FOREGROUND_COLOR + u""";
            font-family: "Open Sans SemiBold";
            font-size: 12px;
        }

        QWidget[h6=true] {
            color: red;
            font-family: "Open Sans SemiBold";
            font-size: 10px;
        }

        QWidget[""" + constants.STYLE_PROPS.MONOSPACE + """=true] {
            font-family: "Consolas", "Liberation Mono", "Courier New", "Courier"
        }

        QLabel[circle-background-icon=true]{
            background: """ + theme.FOREGROUND_COLOR + u""";
            border-radius: 12px;
            min-width: 24px;
            min-height: 24px;

            max-width: 24px;
            max-height: 24px
        }

        QLabel[left-banner=true]{
            padding: 0px;
            background: """ + theme.LEFT_BANNER_BACKGROUND_COLOR + u""";
        }

        QLineEdit {
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            border: 0px solid transparent;
            border-bottom: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 5px 4px;
            margin: 0px 0px 0px 0px;
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QLineEdit[text=\"\"] { /* appearance of placeholder text */
            font-family: "Montserrat Light";
            font-style: italic;
            color: """ + theme.PLACEHOLDER_TEXT_COLOR + u""";
        }

        QLineEdit:hover {
            border-bottom-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QLineEdit:focus {
            border-bottom: 1px solid """ + theme.BRAND_COLOR + u""";
        }

        QLineEdit:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
            border-bottom: 1px solid """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QAbstractSpinBox  {
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            border: 0px solid transparent;
            border-bottom: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 5px 4px;
            margin: 0px 0px 0px 0px;
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractSpinBox:hover {
            border-bottom-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QAbstractSpinBox:focus {
            border-bottom: 1px solid """ + theme.BRAND_COLOR + u""";
        }

        QAbstractSpinBox::up-button {
            margin: 4px 6px 0px 6px;
            border-image: url(""" + str(constants.UP_ARROW_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QAbstractSpinBox::down-button {
            margin: 0px 6px 4px 6px;
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QAbstractSpinBox:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
            border-bottom: 1px solid """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QAbstractSpinBox::up-button:disabled {
            border-image: url(""" + str(constants.UP_ARROW_ICON_OFF_WHITE_PATH.as_posix()) + u""");
        }

        QAbstractSpinBox::down-button:disabled {
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_OFF_WHITE_PATH.as_posix()) + u""");
        }

        QCheckBox {
            spacing: 5px;
        }

        QCheckBox::indicator {
            width: 18px;
            height: 18px;
            background: transparent;
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            border-radius: 2px;
        }

        QCheckBox::indicator:hover {
            border-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QCheckBox::indicator:focus {
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QCheckBox::indicator:unchecked {
            image: url(""" + str(constants.EMPTY_ICON_PATH.as_posix()) + u""");
            background: transparent;
        }

        QCheckBox::indicator:unchecked:hover {
            image: url(""" + str(constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QCheckBox::indicator:unchecked:pressed {
            image: url(""" + str(constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QCheckBox::indicator:unchecked:disabled {
            background: transparent;
            border-color: """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QCheckBox::indicator:checked {
            image: url(""" + str(constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.CHECK_BOX_CHECKED_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED_BORDER_COLOR + u""";
        }

        QCheckBox::indicator:checked:hover {
            background: """ + theme.CHECK_BOX_CHECKED__HOVER_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED__HOVER_BORDER_COLOR + u""";
        }

        QCheckBox::indicator:focus:checked {
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QCheckBox::indicator:checked:disabled {
            image: url(""" + str(constants.CHECKMARK_ICON_OFF_WHITE_PATH.as_posix()) + u""");
            border-color: """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QCheckBox::indicator:focus:checked:hover {
            image: url(""" + str(constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            border-color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QCheckBox::indicator:focus:checked:pressed {
            image: url(""" + str(constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:checked {
            image: url(""" + str(constants.LINK_ICON_UNCHECKED_WHITE_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:unchecked {
            image: url(""" + str(constants.LINK_ICON_BRAND_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:checked:hover {
            image: url(""" + str(constants.LINK_ICON_OFF_WHITE_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:focus:checked {
            image: url(""" + str(constants.LINK_ICON_UNCHECKED_WHITE_PATH.as_posix()) + u""");
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:focus:checked:hover {
            image: url(""" + str(constants.LINK_ICON_UNCHECKED_WHITE_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:focus:checked:pressed {
            image: url(""" + str(constants.LINK_ICON_OFF_WHITE_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:focus:unchecked {
            image: url(""" + str(constants.LINK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:focus:unchecked:hover {
            image: url(""" + str(constants.LINK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            border-color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:focus:unchecked:pressed {
            image: url(""" + str(constants.LINK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:checked:disabled {
            image: url(""" + str(constants.LINK_ICON_DISABLED_WHITE_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[""" + constants.STYLE_PROPS.APP_INFER_CHECKBOX + """=true]::indicator:unchecked:disabled {
            image: url(""" + str(constants.LINK_ICON_DISABLED_BRAND_PATH.as_posix()) + u""");
            background: transparent;
        }
        
        QCheckBox[hidden=true]::indicator {
            border-color: transparent;
            background: transparent;
            image: none;
        }

        QCheckBox[hidden=true]::indicator:checked:disabled {
            border-color: transparent;
            background: transparent;
            image: none;
        }

        QCheckBox[hidden=true]::indicator:unchecked:disabled {
            border-color: transparent;
            background: transparent;
            image: none;
        }


        QComboBox {
            background: transparent;
            color: """ + theme.FOREGROUND_COLOR + u""";
            border: 0px solid transparent;
            border-bottom: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 5px 4px;
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QComboBox:hover {
            border-bottom: 1px solid """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QComboBox:focus {
            border-bottom: 1px solid """ + theme.BRAND_COLOR + u""";
        }

        QComboBox:disabled {
            color: """ + theme.COLOR_DISABLED + u""";
            border-bottom: 1px solid """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QComboBox::drop-down {
            subcontrol-origin: padding;
            subcontrol-position: top right;
            width: 15px;

            margin: 3px 0px;
            padding-left: 5px;
            border-left-width: 1px;
            border-left-color: """ + theme.BORDER_COLOR + u""";
            border-left-style: solid; /* just a single line */
        }

        QComboBox::drop-down:hover {
            border-left-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QComboBox::drop-down:focus {
            border-left-color: """ + theme.BRAND_COLOR + u""";
        }

        QComboBox::drop-down:on {
            border-left-color: """ + theme.BRAND_COLOR + u""";
        }

        QComboBox::down-arrow {
            margin: 2px 2px 2px 2px;
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_WHITE_PATH.as_posix()) + u""");
            image: none;
        }

        QComboBox::down-arrow:disabled {
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_OFF_WHITE_PATH.as_posix()) + u""");
            image: none;
        }

        QAbstractItemView {
            color: """ + theme.FOREGROUND_COLOR + u""";
            background: """ + theme.ABSTRACT_ITEM_VIEW_BACKGROUND + u""";
            alternate-background-color: """ + theme.ABSTRACT_ITEM_VIEW_ALTERNATE_BACKGROUND + u""";
            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
            border: 1px solid transparent;
            border-radius: 2px;
            outline: 0px;
        }

        QAbstractItemView[background-level-2=true] {
            background: """ + theme.BACKGROUND_COLOR_2 + u""";
            alternate-background-color: """ + theme.BACKGROUND_COLOR_1 + u""";
        }

        QAbstractItemView::item{
            height: 28px;
        }

        QAbstractItemView::item:focus{
            border: 0px solid transparent;
            outline: 0px;
        }

        QAbstractItemView::indicator {
            width: 18px;
            height: 18px;
            background: transparent;
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            border-radius: 2px;
        }

        QAbstractItemView::indicator:hover {
            border-color: """ + theme.BORDER_COLOR_HOVER + u""";
        }

        QAbstractItemView::indicator:focus {
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractItemView::indicator:unchecked {
            image: url(""" + str(constants.EMPTY_ICON_PATH.as_posix()) + u""");
            background: transparent;
        }

        QAbstractItemView::indicator:unchecked:hover {
            image: url(""" + str(constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QAbstractItemView::indicator:unchecked:pressed {
            image: url(""" + str(constants.EMPTY_ICON_PATH.as_posix()) + u""");
        }

        QAbstractItemView::indicator:unchecked:disabled {
            border-color: #ebebeb;
        }

        QAbstractItemView::indicator:checked {
            image: url(""" + str(constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.CHECK_BOX_CHECKED_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED_BORDER_COLOR + u""";
        }

        QAbstractItemView::indicator:checked:hover {
            background: """ + theme.CHECK_BOX_CHECKED__HOVER_BACKGROUND_COLOR + u""";
            border-color: """ + theme.CHECK_BOX_CHECKED__HOVER_BORDER_COLOR + u""";
        }

        QAbstractItemView::indicator:focus:checked {
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractItemView::indicator:checked:disabled {
            background: """ + theme.COLOR_DISABLED + u""";
            border-color: """ + theme.BORDER_COLOR_DISABLED + u""";
        }

        QAbstractItemView::indicator:focus:checked:hover {
            image: url(""" + str(constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
            border-color: """ + theme.BRAND_COLOR_HIGHLIGHT + u""";
        }

        QAbstractItemView::indicator:focus:checked:pressed {
            image: url(""" + str(constants.CHECKMARK_ICON_WHITE_PATH.as_posix()) + u""");
            background: """ + theme.BRAND_COLOR + u""";
            border-color: """ + theme.BRAND_COLOR + u""";
        }

        QAbstractItemView[hidden=true]::indicator {
            border: 0px solid transparent;
            background: transparent;
            image: none;
        }

        QAbstractItemView[hidden=true]::indicator:checked:disabled {
            border: 0px solid transparent;
            background: transparent;
        }

        QHeaderView {
            background: transparent;
            border: 0px solid trasparent;
        }

        QHeaderView::section {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
            border-left: 1px solid #6c6c6c;
            border-right: 1px solid #6c6c6c;
            margin-left: 2px;
            padding: 10px 8px;
        }

        QTreeView::branch {
            padding-right: 4px;
            background: transparent
        }

        QTreeView::branch:has-siblings:!adjoins-item {
            background: transparent;
        }

        QTreeView::branch:has-siblings:adjoins-item {
            background: transparent;
        }

        QTreeView::branch:!has-children:!has-siblings:adjoins-item {
            background: transparent;
        }

        QTreeView::branch:closed:has-children:has-siblings {
            image: url(""" + str(constants.RIGHT_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QTreeView::branch:closed:has-children:has-siblings:hover {
            image: url(""" + str(constants.RIGHT_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }

        QTreeView::branch:has-children:!has-siblings:closed {
            image: url(""" + str(constants.RIGHT_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QTreeView::branch:has-children:!has-siblings:closed:hover {
            image: url(""" + str(constants.RIGHT_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }

        QTreeView::branch:open:has-children:has-siblings {
            image: url(""" + str(constants.DOWN_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QTreeView::branch:open:has-children:has-siblings:hover {
            image: url(""" + str(constants.DOWN_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }

        QTreeView::branch:open:has-children:!has-siblings {
            image: url(""" + str(constants.DOWN_ARROW_2_ICON_WHITE_PATH.as_posix()) + u""");
        }

        QTreeView::branch:open:has-children:!has-siblings:hover {
            image: url(""" + str(constants.DOWN_ARROW_2_ICON_BRAND_PATH.as_posix()) + u""");
        }


        QTextEdit:focus{
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
            border: 0px solid transparent;
        }

        QMenu {
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
        }

        QMenu::item {
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
            padding: 7px 20px 7px 30px;
        }

        QMenu::item:hover {
            color: """ + theme.FOREGROUND_COLOR + u""";
        }

        QMenu::item:selected {
            color: """ + theme.WHITE + u""";
            background: """ + theme.BRAND_COLOR + u""";
        }


        QMenu::icon { /* appearance of a 'checked' icon */
            padding: 7px 7px;
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
        }

        QMenu::icon:selected {
            background: """ + theme.BRAND_COLOR + u""";
        }

        QMenu::separator {
            height: 1px;
            background: """ + theme.BORDER_COLOR + u""";
            margin-top: 5px;
            margin-bottom: 5px;
            margin-left: 0px;
            margin-right: 0px;
            border: 0px solid transparent;
        }

        /* appearance of the 'checked' icon */
        QMenu::indicator {
            margin-top: 2px;
            margin-bottom: 2px;
            margin-left: 4px;
            min-width: 16px;
            min-height: 16px;
            background: transparent;
        }

        QMenu::indicator:non-exclusive:unchecked {
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            border-radius: 2px;
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
            image: none;
        }

        QMenu::indicator:non-exclusive:unchecked:selected {
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            image: none;
        }

        QMenu::indicator:non-exclusive:checked {
            border: 1px solid """ + theme.BORDER_COLOR_HOVER + u""";
            border-radius: 2px;
            background: """ + theme.BORDER_COLOR_HOVER + u""";
            image: none;
        }

        QMenu::indicator:non-exclusive:checked:selected {
            border: 1px solid """ + theme.FOREGROUND_COLOR + u""";
            background: """ + theme.FOREGROUND_COLOR + u""";
            image: none;
        }


        QMenuBar::item {
            padding: 3px 10px;
            background: transparent;
        }

        QMenuBar::item:disabled {
            color: #c4c4c4
        }

        QMenuBar::item:selected { /* when selected using mouse or keyboard */
            color: """ + theme.BRAND_COLOR + u""";
        }

        QMenuBar::item:pressed {
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }


        QMenu::item[inPopup]{
            border: 0px;
            min-width: 999px;
        }

        QLabel[user-name-header=true]{
            font-family: "Montserrat Medium";
            font-size: 14px;
        }

        QLabel[invalid=true]{
            color: red;
        }
        
        QTextEdit[background-level-0=true]{
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
        }
        
        QTextEdit[background-level-1=true]{
            background: """ + theme.BACKGROUND_COLOR_1 + u""";
        }
        
        QTextEdit[background-level-2=true]{
            background: """ + theme.BACKGROUND_COLOR_2 + u""";
        }

        QTextEdit[console=true]{
            font-family: "Consolas", "Liberation Mono", "Courier New", "Courier";

            color: """ + theme.WHITE + u""";
            background: """ + theme.CONSOLE_BACKGROUND_COLOR + u""";

            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QPlainTextEdit[console=true]{
            font-family: "Consolas", "Liberation Mono", "Courier New", "Courier";

            color: """ + theme.WHITE + u""";
            background: """ + theme.CONSOLE_BACKGROUND_COLOR + u""";

            selection-color: """ + theme.WHITE + u""";
            selection-background-color: """ + theme.BRAND_COLOR + u""";
        }

        QSplitter::handle {
            image: none;
        }


        QScrollBar:vertical {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            width: 12px;
            padding: 15px 0 15px 0;
        }

        QScrollBar::handle:vertical {
            background: """ + theme.SCROLL_BAR_HANDLE_BACKGROUND + u""";;
            min-height: 20px;
            opacity: 10;
        }

        QScrollBar::add-line:vertical {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            height: 15px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
            border-image: url(""" + str(constants.DOWN_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::sub-line:vertical {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            height: 15px;
            subcontrol-position: top;
            subcontrol-origin: margin;
            border-image: url(""" + str(constants.UP_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
            width: 3px;
            height: 3px;
            background: none;
        }

        QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
            background: none;
        }


        QScrollBar:horizontal {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            height: 12px;
            padding: 0 15px 0 15px;
        }

        QScrollBar::handle:horizontal {
            background: """ + theme.SCROLL_BAR_HANDLE_BACKGROUND + u""";;
            min-width: 20px;
            opacity: 10;
        }

        QScrollBar::add-line:horizontal {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            width: 12px;
            subcontrol-position: bottom left;
            subcontrol-origin: margin;
            border-image: url(""" + str(constants.LEFT_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::sub-line:horizontal {
            background: """ + theme.SCROLL_BAR_BACKGROUND + u""";;
            width: 12px;
            subcontrol-position: top right;
            subcontrol-origin: margin;
            border-image: url(""" + str(constants.RIGHT_ARROW_ICON_OFF_BLACK_PATH.as_posix()) + u""");
            image: none;
        }

        QScrollBar::right-arrow:horizontal, QScrollBar::left-arrow:horizontal {
            width: 3px;
            height: 3px;
            background: none;
        }

        QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
            background: none;
        }


        QToolTip{
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            font-style: normal;
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 7px;
        }

        QLabel[tool-tip=true]{
            font-family: "Montserrat Medium", "open sans";
            font-size: 12px;
            font-style: normal;
            color: """ + theme.FOREGROUND_COLOR_3 + u""";
            background: """ + theme.MENU_BACKGROUND_COLOR + u""";
            border: 1px solid """ + theme.BORDER_COLOR + u""";
            padding: 12px;
        }

        QTabWidget {
            color: red;
            background-color: red;
        }

        QTabWidget::pane {
            color: blue;
            background-color: blue;
        }

        QTabWidget::tab-bar {
            color: green;
            background-color: green;
        }

        QTabBar::tab {
            background-color: """ + theme.TAB_BAR_TAB_BACKGROUND + u""";
        }

        QTabBar::tab:hover {
            background-color: """ + theme.TAB_BAR_TAB_BACKGROUND_HOVERED + u""";
        }

        QTabBar::tab:selected {
            background-color: """ + theme.TAB_BAR_TAB_BACKGROUND_SELECTED + u""";
        }

        QTabBar::tab:selected:hover {
            background-color: """ + theme.TAB_BAR_TAB_BACKGROUND_SELECTED_HOVERED + u""";
        }
        
        QTabBar QToolButton {
            background-color: """ + theme.TAB_BAR_TAB_BACKGROUND + u""";
        }

        QTabWidget > QWidget { background-color: """ + theme.BACKGROUND_COLOR_1 + u"""; }

        QTableView QTableCornerButton::section {
            background: """ + theme.BACKGROUND_COLOR_0 + u""";
        }
        """

    @staticmethod
    def load_fonts() -> None:
        for font_file in Submit2GMUserInterface.CUSTOM_FONT_FILES:
            font_id = QtGui.QFontDatabase.addApplicationFont(str(constants.FONTS_DIRECTORY / font_file))

            if font_id == -1:
                raise ValueError(u"Unable to load font '{}".format(font_file))

            Submit2GMUserInterface.LOADED_FONTS.append(font_id)

    @staticmethod
    def unload_fonts() -> None:
        for font_id in Submit2GMUserInterface.LOADED_FONTS:
            QtGui.QFontDatabase.removeApplicationFont(font_id)
