__all__ = 'Transition'

import typing

from .errors import RejectedTransitionInputError

if typing.TYPE_CHECKING:
    from .project_attribute import ProjectAttribute


class Transition:

    def __init__(self,
                 transition_formula: str,
                 project_attribute: 'ProjectAttribute',
                 rejection_message: typing.Optional[str] = None):

        self._project_attribute = project_attribute
        self._transition_formula = transition_formula

        if rejection_message is None:
            self._rejection_message = "Transition input must satisfy " + self._transition_formula
        else:
            self._rejection_message = rejection_message

    def get_transition_formula(self) -> str:
        return self._transition_formula

    def get_project_attribute(self) -> 'ProjectAttribute':
        return self._project_attribute

    def get_rejection_message(self) -> typing.Optional[str]:
        return self._rejection_message

    def transition(self, transition_input: str) -> 'ProjectAttribute':
        if transition_input is None:
            transition_input = ''

        import re

        if re.match(self.get_transition_formula(), transition_input, re.UNICODE):
            return self.get_project_attribute()

        raise RejectedTransitionInputError()
