__all__ = 'UserContainer'

import typing

from .list_container import ListContainer

if typing.TYPE_CHECKING:
    from .submit2gm_plugin import Submit2gmPlugin
    from .user import User


class UserContainer(ListContainer):

    def __init__(self, plugin: 'Submit2gmPlugin',
                 users: typing.List['User'],
                 default_user: typing.Optional['User'] = None):

        ListContainer.__init__(self, plugin, users)
        self._default_user = None
        self.set_default_user(default_user)

    def append(self, item, focus_new_item = True):
        ListContainer.append(self, item, focus_new_item = focus_new_item)

        if self.get_default_user() is None:
            self.set_default_user(self._items[-1])

    def remove(self, item):

        if item == self._default_user:
            self.set_default_user(None)

        ListContainer.remove(self, item)

    def get_default_user(self):
        return self._default_user

    def set_default_user(self, user):

        if user is None:
            self._default_user = None
        else:
            if not self.contains(user):
                raise ValueError("Can not set an item as default that is not already in the list")

            self._default_user = user

    def set_focused_as_default_user(self):
        self.set_default_user(self.get_focused_item())

    def load_focused_profile(self):
        plugin = self.get_plugin()

        focused_item = self.get_focused_item()

        if focused_item is not None:
            plugin.get_user_interface().load_profile(focused_item)
