__all__ = 'WatchPattern'

import typing


class WatchPattern:

    def __init__(self, remote_watch_pattern: str, local_download_path: str):
        self._remote_watch_pattern = remote_watch_pattern
        self._local_download_path = local_download_path

    def get_remote_watch_pattern(self) -> str:
        return self._remote_watch_pattern

    def set_remote_watch_pattern(self, remote_watch_pattern: str) -> None:
        self._remote_watch_pattern = remote_watch_pattern

    def get_local_download_path(self) -> str:
        return self._local_download_path

    def set_local_download_path(self, local_download_path: str) -> None:
        self._local_download_path = local_download_path

    def serialise(self):
        return '{},{}'.format(self._remote_watch_pattern, self._local_download_path)

    @staticmethod
    def parse_watch_patterns(watch_patterns_str) -> typing.List['WatchPattern']:

        if not watch_patterns_str:
            return []

        components = watch_patterns_str.split(',')

        if len(components) % 2 != 0:
            error_message = "Could not parse watch patterns:\n{0}".format(str(watch_patterns_str))
            raise ValueError(error_message)

        watch_patterns = []

        component_iterator = iter(components)
        for component in component_iterator:
            watch_patterns.append(WatchPattern(component, next(component_iterator)))

        return watch_patterns

    @staticmethod
    def serialise_watch_patterns(watch_patterns: typing.List['WatchPattern']) -> str:
        return ','.join(list(map(lambda x: x.serialise(), watch_patterns)))
