__all__ = 'ComboBox'

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets
import typing

from submit2gm.widgets.input_widgets.input_widget import InputWidget

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class ComboBox(QtWidgets.QComboBox, InputWidget):

    def __init__(self, plugin: 'Submit2gmPlugin',
                 update_items_callback: typing.Callable[[], None] = None,
                 parent: typing.Optional[QtWidgets.QWidget] = None):

        QtWidgets.QComboBox.__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)

        self.update_items_callback=update_items_callback
        self.currentIndexChanged.connect(self._catch_value_change)

        # set strong focus so input does not gain focus from mouse wheel events
        self.setFocusPolicy(QtCore.Qt.StrongFocus)

    def set_update_items_callback(self, update_items_callback: typing.Callable[[], None]) -> None:
        self.update_items_callback = update_items_callback

    def update_items(self) -> None:
        if self.update_items_callback:
            self.update_items_callback()

    def get_value(self) -> str:
        return self.currentData()

    def set_value(self, value: str) -> None:
        index = self.findData(value)
        self.setCurrentIndex(index)

    def wheelEvent(self, event: QtGui.QWheelEvent) -> None:
        if self.hasFocus():
            QtWidgets.QComboBox.wheelEvent(self, event)
        else:
            event.ignore()
