__all__ = 'DoubleSpinBox'

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets
import typing

from submit2gm.widgets.input_widgets.input_widget import InputWidget

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class DoubleSpinBox(QtWidgets.QDoubleSpinBox, InputWidget):

    def __init__(self, plugin: 'Submit2gmPlugin', parent: typing.Optional['QtWidgets.QWidget'] = None):
        QtWidgets.QDoubleSpinBox .__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)

        self.valueChanged.connect(self._catch_value_change)

        # set strong focus so input does not gain focus from mouse wheel events
        self.setFocusPolicy(QtCore.Qt.StrongFocus)

    def get_value(self) -> float:
        return self.value()

    def set_value(self, value: float) -> None:
        if isinstance(value, str):
            value = float(value)

        self.setValue(value)

    def wheelEvent(self, event: QtGui.QWheelEvent) -> None:
        if self.hasFocus():
            QtWidgets.QDoubleSpinBox.wheelEvent(self, event)
        else:
            event.ignore()
