__all__ = 'LineEdit'

from PySide2 import QtWidgets
import re
import typing

from submit2gm.widgets.input_widgets.input_widget import InputWidget

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class LineEdit(QtWidgets.QLineEdit, InputWidget):

    def __init__(self, plugin: 'Submit2gmPlugin', parent=None):
        QtWidgets.QLineEdit.__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)
        self._forbidden_chars_pattern = re.compile(r'((?<!\d)\d(?!\d))')

        self.editingFinished.connect(self._remove_forbidden_characters)
        self.textChanged.connect(self._remove_forbidden_characters)

    def _remove_forbidden_characters(self):
        """
        Replaces some common characters that should never be present in a QLineEdit but which can be pasted into the
        widget.
        """

        cursor_pos = self.cursorPosition()
        text = self.get_value()

        new_chars = []
        new_cursor_pos = 0

        for i, char in enumerate(text):
            if char not in ['\n', '\r', '\t']:
                new_chars.append(char)

                if i < cursor_pos:
                    new_cursor_pos = new_cursor_pos + 1

        self.setText(''.join(new_chars))
        self.setCursorPosition(new_cursor_pos)

        self._catch_value_change()
        self.recalculate_styles()

    def recalculate_styles(self) -> None:
        # required to update the style sheet for the placeholder text style
        self.style().unpolish(self)
        self.style().polish(self)

    def get_value(self) -> str:
        return self.text()

    def set_value(self, value: str) -> None:
        self.setText(value)
