__all__ = 'ListInput'

from PySide2 import QtWidgets
from PySide2 import QtCore
import typing

from submit2gm import constants
from submit2gm.widgets.input_widgets.input_widget import InputWidget

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class ListInput(QtWidgets.QWidget, InputWidget):

    def __init__(self, plugin: 'Submit2gmPlugin', parent: typing.Optional['QtWidgets.QWidget'] = None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)

        """ Create Widgets """
        self.group_box = QtWidgets.QGroupBox()
        self.group_box.setProperty("background-level-2", True)

        self._add_item_btn = QtWidgets.QPushButton(u"+")
        self._add_item_btn.setFlat(True)
        self._add_item_btn.setProperty(constants.STYLE_PROPS.BORDERED, True)

        self._remove_item_btn = QtWidgets.QPushButton(u"-")
        self._remove_item_btn.setFlat(True)
        self._remove_item_btn.setProperty(constants.STYLE_PROPS.BORDERED, True)

        """ Create Layout """
        self._controls_layout = QtWidgets.QVBoxLayout()
        self._controls_layout.setContentsMargins(0, 0, 0, 0)
        self._controls_layout.setAlignment(QtCore.Qt.AlignTop)
        self._controls_layout.addWidget(self._add_item_btn)
        self._controls_layout.addWidget(self._remove_item_btn)

        self._items_layout = QtWidgets.QVBoxLayout()
        self._items_layout.setContentsMargins(0, 0, 0, 0)
        self._items_layout.setAlignment(QtCore.Qt.AlignTop)

        self.hlayout = QtWidgets.QHBoxLayout(self.group_box)
        self.hlayout.setContentsMargins(0, 0, 0, 0)
        self.hlayout.addLayout(self._items_layout)
        self.hlayout.addStretch()
        self.hlayout.addLayout(self._controls_layout)

        self.layout = QtWidgets.QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self.group_box)

        """ Create Connections """

    def minimumSizeHint(self) -> 'QtCore.QSize':
        return QtCore.QSize(0, 0)

    def get_value(self) -> str:
        return ""

    def set_value(self, value: str) -> None:
        pass
