__all__ = ['MainUISwitcher']

from PySide2 import QtCore
from PySide2 import QtWidgets
import typing

from submit2gm import constants
from submit2gm.plugin_accessor import PluginAccessor

from .submission_view_widget import SubmissionViewWidget
from .resizable_stacked_widget import ResizableStackedWidget
from .top_banner import TopBanner

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class MainUISwitcher(QtWidgets.QWidget, PluginAccessor):

    def __init__(self, plugin: 'Submit2gmPlugin', parent: typing.Optional['QtWidgets.QWidget'] = None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        PluginAccessor.__init__(self, plugin)

        self.job_preset_container_view_index = None
        self.pack_current_scene_view_index = None
        self.upload_current_scene_view_index = None
        self.upload_packed_project_view_index = None
        self.submission_view_index = None

        """ Create Widgets """
        # top banner
        self.top_banner = TopBanner(plugin)

        self._stacked_widget = ResizableStackedWidget()
        self._stacked_widget.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

        """ Create Layouts """
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

        self.top_bar_layout = QtWidgets.QHBoxLayout()
        self.top_bar_layout.addWidget(self.top_banner)
        self.top_bar_layout.setAlignment(QtCore.Qt.AlignHCenter)

        self._layout = QtWidgets.QVBoxLayout(self)
        self._layout.setAlignment(QtCore.Qt.AlignTop)
        self._layout.setContentsMargins(constants.MARGIN_SIZE,
                                        constants.MARGIN_SIZE,
                                        constants.MARGIN_SIZE,
                                        constants.MARGIN_SIZE)
        self._layout.addLayout(self.top_bar_layout)
        self._layout.addSpacing(10)
        self._layout.addWidget(self._stacked_widget)

        #self.display_job_preset_container()
        #self.display_pack_current_scene_view()
        #self.display_upload_current_scene_view()
        #self.display_upload_packed_project_view()
        self.display_submission_view()

    def get_submission_view_widget(self) -> 'SubmissionViewWidget':
        if self.submission_view_index is not None:
            return self._stacked_widget.widget(self.submission_view_index)

    def get_current_widget(self) -> typing.Optional[QtWidgets.QWidget]:
        return self._stacked_widget.currentWidget()

    def display_job_preset_container(self) -> None:
        return

        # hide job preset container for v1
        """
        if self.job_preset_container_view_index is None:
            widget = self.get_plugin().get_preferences_container().get_job_preset_container()
            index = self._stacked_widget.addWidget(widget)
            self.job_preset_container_view_index = index

        self._stacked_widget.setCurrentIndex(self.job_preset_container_view_index)

        # redraw the current focused job preset in the job preset container
        job_preset_container_widget = self._stacked_widget.currentWidget()
        current_item = job_preset_container_widget.job_preset_to_tree_item(job_preset_container_widget.get_focused_item())
        job_preset_container_widget.current_item_changed(current_item, current_item)
        """

    def display_pack_current_scene_view(self) -> None:
        return

        """
        if self.pack_current_scene_view_index is None:
            widget = PackCurrentSceneWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.pack_current_scene_view_index = index

        self._stacked_widget.setCurrentIndex(self.pack_current_scene_view_index)
        """

    def display_upload_current_scene_view(self) -> None:
        return

        """
        if self.upload_current_scene_view_index is None:
            widget = UploadCurrentSceneWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.upload_current_scene_view_index = index

        self._stacked_widget.setCurrentIndex(self.upload_current_scene_view_index)
        """

    def display_upload_packed_project_view(self) -> None:
        return

        """
        if self.upload_packed_project_view_index is None:
            widget = UploadPackedProjectWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.upload_packed_project_view_index = index

        self._stacked_widget.setCurrentIndex(self.upload_packed_project_view_index)
        """

    def display_submission_view(self) -> None:
        if self.submission_view_index is None:
            widget = SubmissionViewWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.submission_view_index = index

        self._stacked_widget.setCurrentIndex(self.submission_view_index)
