__all__ = "LeftBanner"

from PySide2 import QtCore
from PySide2 import QtWidgets
from PySide2 import QtGui
import typing

from submit2gm import ui_constants, qt_utils
from submit2gm.plugin_accessor import PluginAccessor

from .pixmap_label import PixmapLabel

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class TopBanner(PixmapLabel, PluginAccessor):
    BANNER_HEIGHT = 74

    SMALL_BANNER_IMAGE = QtGui.QPixmap(ui_constants.DEFAULT_TOP_BANNER_SHORT_IMAGE)
    WIDE_BANNER_IMAGE = QtGui.QPixmap(ui_constants.DEFAULT_TOP_BANNER_WIDE_IMAGE)
    BREAK_POINT = 800

    def __init__(self, plugin: 'Submit2gmPlugin', parent: typing.Optional[QtWidgets.QWidget] = None):
        PixmapLabel.__init__(self, parent=parent)
        PluginAccessor.__init__(self, plugin)

        self.setText(u"    60 machines not enough?"
                     "\n    Please contact us if you need more!"
                     "\n    Let's render...")

        self.setPixmap(TopBanner.SMALL_BANNER_IMAGE)
        self.setFixedHeight(TopBanner.BANNER_HEIGHT)
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        self.setAlignment(QtCore.Qt.AlignCenter)
        self.setCursor(QtCore.Qt.PointingHandCursor)
        pal = self.palette()
        pal.setColor(self.foregroundRole(), QtGui.QColor(255, 0, 0))
        self.setPalette(pal)
        self.setStyleSheet("color: white;")  # used to style the font color of the top banner

        banner_font = QtGui.QFont("Open Sans")
        banner_font.setBold(True)
        banner_font.setPixelSize(30)
        self.setFont(banner_font)
        self.clicked.connect(self.handle_clicked_signal)

    def handle_clicked_signal(self) -> None:
        qt_utils.open_url(self.get_plugin().get_user_interface().get_left_banner_link())

    def resizeEvent(self, event: 'QtGui.QResizeEvent'):
        width = event.size().width()

        if width > TopBanner.BREAK_POINT:
            self.setPixmap(TopBanner.WIDE_BANNER_IMAGE)
        else:
            self.setPixmap(TopBanner.SMALL_BANNER_IMAGE)

        QtWidgets.QWidget.resizeEvent(self, event)
