__all__ = 'ApplicationAttributeSource'

from abc import abstractmethod
import typing

if typing.TYPE_CHECKING:
    from .plugin import Plugin
    from .attribute_types import EnumItem


class ApplicationAttributeSource:

    def __init__(self, plugin: 'Plugin'):
        self._plugin = plugin

    def get_plugin(self) -> 'Plugin':
        return self._plugin

    @abstractmethod
    def get_attribute_value(self, app: str, version: str, key: str) -> any:
        raise NotImplementedError

    @abstractmethod
    def get_enum_items(self, app: str, version: str, key: str) -> (typing.List['EnumItem'], int):
        raise NotImplementedError

    def _get_enum_index(self, app: str, version: str, key: str, items: typing.List['EnumItem']) -> int:
        """
        Gets the current attribute value and returns it's index in the list of possible values. Defaults to 0 if the
        current value is not found in the provided items list.
        """

        current_value = self.get_attribute_value(app, version, key)
        try:
            # convert the enum items list into a list of only the keys
            keys = list(map(lambda x: x.get_key(), items))

            # try to get the index of the currently selected key
            default_index = keys.index(current_value)
        except ValueError:
            # otherwise just use the first index
            default_index = 0

        return default_index
