__all__ = ['JobPresetAttribute']


from gm_meta_plugin.plugin_accessor import PluginAccessor

import abc
import typing

if typing.TYPE_CHECKING:
    from gm_meta_plugin.job_attribute import JobAttribute
    from gm_meta_plugin.job_preset import JobPreset
    from gm_meta_plugin.plugin import Plugin


class JobPresetAttribute(PluginAccessor):

    def __init__(self,
                 plugin: 'Plugin',
                 id: 'str',
                 job_attribute: 'JobAttribute'):

        PluginAccessor.__init__(self, plugin)
        self._id = id
        self._job_attribute = job_attribute

    @abc.abstractmethod
    def get_parent_preset(self) -> 'JobPreset':
        raise NotImplementedError

    def get_job_attribute(self) -> 'JobAttribute':
        return self._job_attribute

    def get_id(self):
        return self._id

    def get_key(self):
        return self.get_job_attribute().get_attribute().get_key()

    def get_optional_parent(self) -> 'JobPresetAttribute':
        optional_parent = self.get_job_attribute().get_optional_parent()

        if optional_parent is None:
            raise ValueError("Cannot get optional parent as job preset attribute '' does not have "
                             "one.".format(self.get_id()))

        return self.get_parent_preset().get_job_preset_attribute_with_job_attribute_id(optional_parent)

    @abc.abstractmethod
    def get_value(self) -> any:
        raise NotImplementedError

    @abc.abstractmethod
    def should_skip(self) -> bool:
        raise NotImplementedError


