__all__ = 'Plugin'

from abc import abstractmethod
import typing

if typing.TYPE_CHECKING:
    from . import PluginVersion, LoggingCoordinator, PreferencesContainer, APIClient, UserInterface, \
        RemoteProjectContainer, ApplicationAttributeSource, FactoryCollection, JobPresetValidator, \
        PluginUtils, PackedSceneBuilder


class Plugin(object):

    @abstractmethod
    def get_name(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def get_version(self) -> 'PluginVersion':
        raise NotImplementedError

    @abstractmethod
    def get_logging_coordinator(self) -> 'LoggingCoordinator':
        raise NotImplementedError

    @abstractmethod
    def get_preferences_container(self) -> 'PreferencesContainer':
        raise NotImplementedError

    @abstractmethod
    def get_api_client(self) -> 'APIClient':
        raise NotImplementedError

    @abstractmethod
    def get_user_interface(self) -> 'UserInterface':
        raise NotImplementedError

    @abstractmethod
    def get_remote_project_container(self) -> 'RemoteProjectContainer':
        raise NotImplementedError

    @abstractmethod
    def get_application_attribute_source(self) -> 'ApplicationAttributeSource':
        raise NotImplementedError

    @abstractmethod
    def get_packed_scene_builder(self) -> 'PackedSceneBuilder':
        raise NotImplementedError

    @abstractmethod
    def get_factory_collection(self) -> 'FactoryCollection':
        raise NotImplementedError

    @abstractmethod
    def get_job_preset_validator(self) -> 'JobPresetValidator':
        raise NotImplementedError

    def get_plugin_utils(self) -> 'PluginUtils':
        raise NotImplementedError

    @abstractmethod
    def get_supported_applications(self) -> typing.List[str]:
        raise NotImplementedError
