__all__ = 'Project'

from abc import abstractmethod
import typing

if typing.TYPE_CHECKING:
    from . import WatchPattern

class Project:

    def __init__(self,
                 name,
                 root_dir,
                 files,
                 attributes):
        self._name = name
        self._root_dir = root_dir
        self._files = files
        self._attributes = attributes
        self._watch_patterns: typing.List['WatchPattern'] = []

    def get_name(self):
        return self._name

    def set_name(self, name):
        self._name = name

    def get_attributes(self):
        return self._attributes

    def get_attribute(self, key):
        return self._attributes[key]

    def set_attribute(self, key, value):
        self._attributes[key] = value

    @abstractmethod
    def exists(self):
        raise NotImplementedError

    @abstractmethod
    def delete(self):
        raise NotImplementedError

    def get_root_dir(self):
        return self._root_dir

    def get_files(self):
        return self._files

    def add_files(self, files):
        self._files = self._files | files

    def set_files(self, files):
        self._files = files

    def get_watch_patterns(self) -> typing.List['WatchPattern']:
        return self._watch_patterns

    def set_watch_patterns(self, watch_patterns: typing.List['WatchPattern']) -> None:
        self._watch_patterns = watch_patterns

    @abstractmethod
    def get_size(self):
        """
        Gets the size of the project in bytes.

        :return: The size of the project in bytes.
        :rtype: int
        """
        raise NotImplementedError
