__all__ = 'ProjectDefinitionSchema'

import typing

from gm_meta_plugin.plugin_accessor import PluginAccessor

if typing.TYPE_CHECKING:
    from gm_meta_plugin.project_attribute import ProjectAttribute


class ProjectDefinitionSchema(PluginAccessor):

    def __init__(self, plugin, root_project_attribute, project_attributes):
        PluginAccessor.__init__(self, plugin=plugin)

        if root_project_attribute not in project_attributes:
            raise ValueError("Root project attribute not in project attribute list")

        self._root_project_attribute = root_project_attribute
        self._project_attributes = project_attributes

    def get_root_project_attribute(self) -> 'ProjectAttribute':
        return self._root_project_attribute

    def get_project_attributes(self) -> typing.List['ProjectAttribute']:
        return self._project_attributes

    def get_project_attribute_with_id(self, project_attribute_id) -> 'ProjectAttribute':
        for project_attribute in self.get_project_attributes():
            if project_attribute.get_id() == project_attribute_id:
                return project_attribute

        raise ValueError("No project attribute with id '" + project_attribute_id + "'")
