__all__ = 'ProjectInstanceBuilder'

from abc import abstractmethod
import sys

from gm_meta_plugin.project_definition_schema import ProjectDefinitionSchema
from gm_meta_plugin.attribute_types import AttributeType
from gm_meta_plugin.errors.application_attribute_not_found import ApplicationAttributeNotFound
from gm_meta_plugin.errors.cli_prompt_not_enabled_error import CLIPromptNotEnabledError

class ProjectInstanceBuilder(ProjectDefinitionSchema):

    def __init__(self, plugin, root_project_attribute_instance, project_attribute_instances):
        ProjectDefinitionSchema.__init__(self, plugin, root_project_attribute_instance, project_attribute_instances)

    def update_states(self, project_name=None, application_attribute_source=None, infer_from_cmd=False):

        if application_attribute_source is None:
            application_attribute_source = self.get_plugin().get_application_attribute_source()

        project_attribute = self.get_root_project_attribute()

        if project_name is None:
            if infer_from_cmd:
                try:
                    project_name = project_attribute.get_attribute().get_argv_value(self.get_plugin(), key=project_attribute.get_id())
                except CLIPromptNotEnabledError as e:
                    default_value = project_attribute.get_attribute().get_default_value()
                    if default_value is None or default_value == "":
                        raise e

                    project_name = default_value

            else:
                project_name = u""

        # set project name
        project_attribute.set_value(project_name)

        project_attribute = project_attribute.transition()
        while True:
            attribute = project_attribute.get_attribute()

            if attribute.get_type() == AttributeType.NULL:
                break

            if infer_from_cmd:
                try:
                    app_inferred_value = attribute.get_argv_value(self.get_plugin(), key=project_attribute.get_id())
                except CLIPromptNotEnabledError as e:
                    default_value = attribute.get_default_value()
                    if default_value is None or default_value == "":
                        raise e

                    app_inferred_value = default_value

            else:
                # attempt to set the project attributes value to the application inferred value
                try:
                    app_inferred_value = application_attribute_source.get_attribute_value(None,
                                                                                          None,
                                                                                          project_attribute.get_attribute().get_key())
                except ApplicationAttributeNotFound as e:
                    # if the application source does not recognise this attribute key then use the default value
                    app_inferred_value = project_attribute.get_attribute().get_default_value()

                except ValueError as e:
                    if str(e) != "No command port servers":
                        raise e

                    app_inferred_value = project_attribute.get_attribute().get_default_value()

            project_attribute.set_value(app_inferred_value)
            project_attribute = project_attribute.transition(force_transition=True)

    @abstractmethod
    def get_project_attributes(self, attributes=None, force_transition=False, is_remote=False):
        raise NotImplementedError
