__all__ = ['StaticJobPresetAttribute']

from .job_preset_attribute import JobPresetAttribute

import typing

if typing.TYPE_CHECKING:
    from gm_meta_plugin.job_attribute import JobAttribute
    from gm_meta_plugin.dynamic_job_preset import StaticJobPreset
    from gm_meta_plugin.plugin import Plugin


class StaticJobPresetAttribute(JobPresetAttribute):

    def __init__(self,
                 plugin: 'Plugin',
                 parent_preset: 'StaticJobPreset',
                 job_attribute: 'JobAttribute',
                 has_value: bool,
                 value: 'any',
                 should_skip: bool):

        id = parent_preset.get_id() + '_' + job_attribute.get_attribute().get_key()
        JobPresetAttribute.__init__(self, plugin, id, job_attribute)

        self._parent_preset = parent_preset
        self._job_attribute = job_attribute
        self._has_value = has_value
        self._value = value
        self._should_skip = should_skip

    def get_parent_preset(self) -> 'StaticJobPreset':
        return self._parent_preset

    def has_value(self) -> bool:
        """
        A StaticJobPresetAttribute may or may not have a value set. For example if the DynamicJobPresetAttribute from
        which this StaticJobPresetAttribute was created from did not meet it's dependencies then no value will be set.
        """
        return self._has_value

    def get_value(self) -> any:
        if not self.has_value():
            raise ValueError("StaticJobPresetAttribute ('{}') does not have a value".format(
                self.get_job_attribute().get_id()))

        return self._value

    def should_skip(self) -> bool:
        return self._should_skip

