__all__ = 'Transition'

from .errors import RejectedTransitionInputError


class Transition:

    def __init__(self, transition_formula, project_attribute, rejection_message = None):

        self._project_attribute = project_attribute
        self._transition_formula = transition_formula

        if rejection_message is None:
            self._rejection_message = "Transition input must satisfy " + self._transition_formula
        else:
            self._rejection_message = rejection_message

    def get_transition_formula(self):
        return self._transition_formula

    def get_project_attribute(self):
        return self._project_attribute

    def get_rejection_message(self):
        return self._rejection_message

    def transition(self, input):
        if input is None:
            input = ''

        import re

        if re.match(self.get_transition_formula(), input, re.UNICODE):
            return self.get_project_attribute()

        raise RejectedTransitionInputError
