__all__ = 'APIClient'

import json

import gm_meta_plugin
import gridmarkets
from gm_meta_plugin.gridmarkets import GridMarketsAPIClient


class QtAPIClient(GridMarketsAPIClient):

    def sign_in_with_envoy(self):
        self._log.info("Signing in with envoy")
        http_client = gridmarkets.http_client.HttpClient()

        method = "get"
        url = gridmarkets.envoy_client.API_BASE + "/auth"

        try:
            resp = http_client.request(method, url)
        except gridmarkets.errors.APIError as e:
            e_msg = e.message

            # pretty error message for common errors
            if e_msg.startswith(u"Unexpected error communicating with Envoy.Please start Envoy"):
                e_msg = u"Unable to connect to Envoy."

            raise gm_meta_plugin.errors.APIClientError(message=e_msg)
        except Exception as e:
            raise gm_meta_plugin.errors.APIClientError(message=e.message)

        # Check that the user is signed into Envoy
        try:
            content = resp.json()
        except json.decoder.JSONDecodeError:
            raise gm_meta_plugin.errors.NotSignedInError(message=u"Please sign-in to Envoy.")

        user_name = content['Username']

        if not user_name:
            raise gm_meta_plugin.errors.InvalidUserError(u"Please sign-in to Envoy.")

        # create user
        user = gm_meta_plugin.User(user_name)

        self.sign_in(user, skip_validation=True)
