from PySide2 import QtWidgets, QtCore, QtGui


class ElidedLabel(QtWidgets.QFrame):
    elisionChanged = QtCore.Signal(bool)

    def __init__(self, text, parent=None):
        QtWidgets.QFrame.__init__(self, parent=parent)
        self.content = text
        self.elided = False
        self._color = QtGui.QColor("white")
        self._alignment = QtCore.Qt.AlignVCenter

    def getColor(self):
        return self._color

    def setColor(self, color):
        if isinstance(color, str) or isinstance(color, unicode):
            self._color = QtGui.QColor(color)
        else:
            self._color = color


    def setText(self, newText):
        self.content = newText
        self.update()

    def text(self):
        return self.content

    def alignment(self):
        return self._alignment

    def setAlignment(self, alignment):
        self._alignment = alignment

    def isElided(self):
        return self.elided

    def paintEvent(self, event):
        QtWidgets.QFrame.paintEvent(self, event)

        painter = QtGui.QPainter(self)
        painter.setPen(self._color)
        fontMetrics = painter.fontMetrics()
        rect = self.rect()

        didElide = False
        textLayout = QtGui.QTextLayout(self.content, painter.font())
        textLayout.beginLayout()

        while True:
            line = textLayout.createLine()

            if not line.isValid():
                break

            line.setLineWidth(self.width())
            elided_line = fontMetrics.elidedText(self.content, QtCore.Qt.ElideRight, self.width())
            painter.drawText(rect, int(self._alignment), elided_line)
            line = textLayout.createLine()
            didElide = line.isValid()

        textLayout.endLayout()

        if didElide != self.elided:
            self.elided = didElide
            self.elisionChanged.emit(didElide)