__all__ = 'Header'

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

import gm_meta_plugin

from gm_qt_plugin import qt_constants, qt_utils
from gm_qt_plugin.qt_none import QtNone
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor
from gm_qt_plugin.worker_thread import WorkerThread
from gm_qt_plugin.exception_wrapper import ExceptionWrapper

class Header(QtWidgets.QWidget, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        user_interface = plugin.get_user_interface()
        user_interface.set_style_sheet(self)

        self._is_polling_cps: bool = False

        def display_job_preset_container():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_job_preset_container()

        def display_remote_project_container():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_remote_project_container()

        def display_import_existing_remote_projects():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_import_existing_remote_projects()

        def display_pack_current_scene_view():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_pack_current_scene_view()

        def display_upload_current_scene_view():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_upload_current_scene_view()

        def display_upload_packed_project_view():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_upload_packed_project_view()

        def display_submission_view():
            main_window = self.get_plugin().get_user_interface().get_main_window()
            main_window.get_main_ui_widget().get_main_ui_switcher().display_submission_view()

        # options menu
        _toggle_theme_action = QtWidgets.QAction(u"Toggle Theme", self)
        _toggle_theme_action.triggered.connect(user_interface.toggle_theme)

        self.options_menu = QtWidgets.QMenu(u"Options")
        self.options_menu.addAction(_toggle_theme_action)

        # help menu
        _open_faq_action = QtWidgets.QAction(u"FAQ", self)
        _open_faq_action.setIcon(qt_constants.INFO_ICON_OFF_WHITE)
        _open_faq_action.triggered.connect(lambda: qt_utils.open_url(qt_constants.FAQ_URL))

        # main_product_name = plugin.get_supported_applications()[0].title()
        # _open_product_page_action = QtWidgets.QAction(main_product_name, self)
        # _open_product_page_action.setIcon(qt_constants.INFO_ICON_OFF_WHITE)
        # _open_product_page_action.triggered.connect(lambda: qt_utils.open_url(user_interface.get_help_url()))

        _open_security_page = QtWidgets.QAction(u"Security", self)
        _open_security_page.setIcon(qt_constants.INFO_ICON_OFF_WHITE)
        _open_security_page.triggered.connect(lambda: qt_utils.open_url(qt_constants.SECURITY_URL))

        _open_envoy_logs = QtWidgets.QAction(u"Envoy Logs", self)
        _open_envoy_logs.setIcon(qt_constants.INFO_ICON_OFF_WHITE)
        _open_envoy_logs.triggered.connect(lambda: qt_utils.open_url(qt_constants.ENVOY_LOGS_URL))

        _save_plugin_logs_action = QtWidgets.QAction(u"Save Plugin Logs", self)
        _save_plugin_logs_action.setIcon(qt_constants.NEW_FILE_ICON_OFF_WHITE)
        _save_plugin_logs_action.triggered.connect(lambda: qt_utils.save_logs_to_file(self.get_plugin()))

        def show_support_popup():
            msg_box = QtWidgets.QMessageBox()
            msg_box.setText("Please contact " + qt_constants.SUPPORT_EMAIL + " for support.")
            user_interface.set_style_sheet(msg_box)
            msg_box.exec_()

        def show_version_popup():
            msg_box = QtWidgets.QMessageBox()

            name = plugin.get_name()
            version = plugin.get_version().to_string()
            msg_box.setText(u"{0} version {1}".format(name, version))

            user_interface.set_style_sheet(msg_box)

            msg_box.exec_()

        _show_support_popup_action = QtWidgets.QAction(u"Support", self)
        _show_support_popup_action.setIcon(qt_constants.EMPTY_ICON)
        _show_support_popup_action.triggered.connect(show_support_popup)

        _show_about_popup_action = QtWidgets.QAction(u"About", self)
        _show_about_popup_action.setIcon(qt_constants.EMPTY_ICON)
        _show_about_popup_action.triggered.connect(show_version_popup)

        self.help_menu = QtWidgets.QMenu(u"Help")
        self.help_menu.addAction(_open_faq_action)
        #self.help_menu.addAction(_open_product_page_action)
        self.help_menu.addAction(_open_security_page)
        self.help_menu.addSeparator()
        self.help_menu.addAction(_open_envoy_logs)
        self.help_menu.addAction(_save_plugin_logs_action)
        self.help_menu.addSeparator()
        self.help_menu.addAction(_show_support_popup_action)
        self.help_menu.addSeparator()
        self.help_menu.addAction(_show_about_popup_action)

        # misc menu
        action = QtWidgets.QAction(u"Import Existing Remote Projects", self)
        action.setIcon(qt_constants.EMPTY_ICON)
        action.triggered.connect(display_import_existing_remote_projects)

        self.misc_menu = QtWidgets.QMenu("Misc")
        self.misc_menu.addAction(action)

        # pack menu
        action = QtWidgets.QAction(u"Pack Current Scene", self)
        action.setIcon(qt_constants.EMPTY_ICON)
        action.triggered.connect(display_pack_current_scene_view)

        self.pack_menu = QtWidgets.QMenu("Pack")
        self.pack_menu.addAction(action)
        self.pack_menu.setEnabled(False)

        # upload menu
        upload_current_scene_action = QtWidgets.QAction(u"Upload Current Scene", self)
        upload_current_scene_action.setIcon(qt_constants.EMPTY_ICON)
        upload_current_scene_action.triggered.connect(display_upload_current_scene_view)

        upload_packed_project_action = QtWidgets.QAction(u"Upload Packed Project", self)
        upload_packed_project_action.setIcon(qt_constants.EMPTY_ICON)
        upload_packed_project_action.triggered.connect(display_upload_packed_project_view)

        self.upload_menu = QtWidgets.QMenu("Upload")
        self.upload_menu.addAction(upload_current_scene_action)
        self.upload_menu.addAction(upload_packed_project_action)

        display_submit_layout_action = QtWidgets.QAction(u"Submit", self)
        display_submit_layout_action.triggered.connect(display_submission_view)

        """ Create Widgets """
        self._menu_bar = QtWidgets.QMenuBar()

        # hide additional menus for v1 of the plugin
        # self._menu_bar.addAction(display_submit_layout_action)
        # self._menu_bar.addMenu(self.upload_menu)
        # self._menu_bar.addMenu(self.pack_menu)
        # self._menu_bar.addMenu(self.misc_menu)

        self._menu_bar.addMenu(self.help_menu)
        self._menu_bar.setFixedWidth(55)
        self._menu_bar.setContentsMargins(0, 0, 0, 0)

        self.status_indicator = HeaderStatusIndicator(self)
        self.status_indicator.hide()  # status indicator starts hidden

        self.view_remote_projects_btn = QtWidgets.QPushButton("Projects")
        self.view_remote_projects_btn.setFlat(True)
        self.view_remote_projects_btn.clicked.connect(display_remote_project_container)
        self.view_remote_projects_btn.hide()  # hide for v1 of the plugin

        self.view_job_presets_btn = QtWidgets.QPushButton("Job Presets")
        self.view_job_presets_btn.setFlat(True)
        self.view_job_presets_btn.clicked.connect(display_job_preset_container)
        self.view_job_presets_btn.hide()  # hide for v1 of the plugin

        self.update_available_credits_btn = QtWidgets.QPushButton("")
        self.update_available_credits_btn.setFlat(True)
        self.update_available_credits_btn.clicked.connect(self._update_credits_slot)
        self._set_available_credits()

        self.user_popup_btn = UserPopupButton(plugin)

        """ Create Layouts """
        self.layout = QtWidgets.QHBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self._menu_bar)
        self.layout.addWidget(self.status_indicator)
        self.layout.addStretch()
        self.layout.addWidget(self.view_remote_projects_btn)
        self.layout.addWidget(self.view_job_presets_btn)
        self.layout.addWidget(self.update_available_credits_btn)
        self.layout.addWidget(self.user_popup_btn)
        self.layout.setProperty(qt_constants.STYLE_PROPS.HEADER, True)

        user_interface.set_style_sheet(self.upload_menu)
        user_interface.set_style_sheet(self.pack_menu)
        user_interface.set_style_sheet(self.misc_menu)
        user_interface.set_style_sheet(self.help_menu)
        user_interface.set_style_sheet(self.options_menu)

        """ Create Connections """
        self._cps_heartbeat_timmer = QtCore.QTimer(self)
        self._cps_heartbeat_timmer.setInterval(qt_constants.CPS_HEARTBEAT_TIMER_INTERVAL)
        self._cps_heartbeat_timmer.setSingleShot(True)
        self._cps_heartbeat_timmer.timeout.connect(self._cps_heartbeat_slot)
        self._cps_heartbeat_timmer.start()

        user_interface.operation_started.connect(self.on_operation_started)
        user_interface.operation_finished.connect(self.on_operation_finished)

    def _update_credits_slot(self) -> None:
        log = self.get_plugin().get_logging_coordinator().get_logger(u"_update_credits_slot")
        plugin = self.get_plugin()
        api_client = plugin.get_api_client()
        user_interface = plugin.get_user_interface()
        main_window = user_interface.get_main_window()

        # break timer loop when main window is closed
        if main_window.is_closed() or not api_client.is_user_signed_in():
            return

        worker_thread = user_interface.get_worker_thread(u"Updating account credits...")

        target = api_client.get_user_info

        args = tuple()

        kwargs = {
            'ignore_cache': True
        }

        log.debug(u"process api_client.get_available_credits")
        worker_thread.process(target, args, kwargs, self._update_available_credits_handler)

    def _update_available_credits_handler(self, user_info):
        self._set_available_credits()
        return True

    def _cps_heartbeat_logic(self) -> None:
        log = self.get_plugin().get_logging_coordinator().get_logger(u"_cps_heartbeat_handler")
        log.debug("_cps_heartbeat_logic: ")

        plugin = self.get_plugin()

        cps = plugin.get_active_command_port_server()

        # Check the cps is still running
        if not cps.is_still_running():
            raise gm_meta_plugin.errors.CommandPortServerError("Command port server is no longer running")

    def _cps_heartbeat_slot(self):
        log = self.get_plugin().get_logging_coordinator().get_logger(u"_cps_heartbeat_slot")

        plugin = self.get_plugin()
        user_interface = plugin.get_user_interface()

        self._worker_thread = WorkerThread(self.get_plugin(), parent=self)

        target = self._cps_heartbeat_logic

        args = tuple()

        kwargs = {}

        log.debug(u"cps heartbeat")
        self._is_polling_cps = True
        self._worker_thread.process(target, args, kwargs, self._cps_heartbeat_handler)

    def is_polling_cps(self) -> bool:
        return self._is_polling_cps

    def _cps_heartbeat_handler(self, result):
        log = self.get_plugin().get_logging_coordinator().get_logger(u"_cps_heartbeat_handler")
        self._is_polling_cps = False

        log.debug("_cps_heartbeat_handler result type: " + str(type(result)))
        log.debug("_cps_heartbeat_handler result: " + str(result))

        if isinstance(result, QtNone):
            self._cps_heartbeat_timmer.start()
            return True


        if isinstance(result, ExceptionWrapper):
            exception = result.get_exception()
            log.debug(str(exception))

            if isinstance(exception, ValueError):
                # If there are no active command port servers then stop polling. This should only happen if submit2gm
                # was reset in which case the polling will restart when the main window is recreated.
                return True

            elif isinstance(exception, gm_meta_plugin.errors.CommandPortServerError):

                plugin = self.get_plugin()

                # Remove all old job presets
                job_preset_container = plugin.get_preferences_container().get_job_preset_container()
                job_preset_container.reset()

                # Remove the old job preset from the ui
                plugin.get_user_interface().get_main_switcher_ui().get_submission_view_widget().update_job_options()

                plugin.get_api_client().sign_out()
                plugin.shutdown_command_port_servers()

                main_window = plugin.get_user_interface().get_main_window()
                main_window.display_signed_out_ui()
                logging_coordinator = plugin.get_logging_coordinator()
                logger = logging_coordinator.get_logger(__name__)

                if main_window.isVisible():
                    qt_utils.log_error(self, logger, exception.user_message)

                return True

        return False

    def _set_available_credits(self):
        """ gets the cached available credits from the api client and update the credits label """
        log = self.get_plugin().get_logging_coordinator().get_logger(u"_set_available_credits")

        plugin = self.get_plugin()
        api_client = plugin.get_api_client()

        # use ignore_cache=False so that we don't re-fetch the credits value and block the ui
        available_credits = round(api_client.get_user_info(ignore_cache=False).get_credits_available(),
                                  qt_constants.AVAILABLE_CREDITS_DECIMAL_PLACES)

        formatted_credits_text = u"{0:.2f}{1}".format(available_credits, qt_constants.AVAILABLE_CREDITS_SUFFIX)

        log.debug(u"Setting available credits label text to '{0}'".format(formatted_credits_text))
        self.update_available_credits_btn.setText(formatted_credits_text)

    def get_status_indicator(self):
        return self.status_indicator

    def on_operation_started(self):
        user_interface = self.get_plugin().get_user_interface()
        self.status_indicator.set_status_message(user_interface.get_running_operation_message())
        self.status_indicator.show()

    def on_operation_finished(self):
        self.status_indicator.hide()

class HeaderStatusIndicator(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super(HeaderStatusIndicator, self).__init__(parent)

        spinner_movie = QtGui.QMovie(str(qt_constants.SPINNER_WHITE_2D2D2D_GIF_PATH), parent=self)
        spinner_movie.setCacheMode(QtGui.QMovie.CacheAll)
        spinner_movie.start()

        self.spinner = QtWidgets.QLabel(u"")
        self.spinner.setMovie(spinner_movie)
        self.status_label = QtWidgets.QLabel("Status message")

        self.layout = QtWidgets.QHBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self.spinner)
        self.layout.addWidget(self.status_label)

    def get_status_message(self):
        return self.status_label.text()

    def set_status_message(self, message):
        self.status_label.setText(message)


class UserPopupButton(QtWidgets.QToolButton, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        auth_email = plugin.get_api_client().get_signed_in_user().get_auth_email()

        QtWidgets.QToolButton.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        self.setText(auth_email)
        self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.setIcon(qt_constants.USER_ICON_OFF_WHITE)
        self.setCursor(QtCore.Qt.PointingHandCursor)

        self.pop_up_window = PopUpWindow(plugin, self)

    def close_popup(self):
        self.pop_up_window.hide()

    def mousePressEvent(self, mouseEvent):
        # set the email
        user = self.get_plugin().get_api_client().get_signed_in_user()
        user_email = user.get_auth_email() if user else ""

        self.pop_up_window.email_label.setText(user_email)

        self.pop_up_window.show()

        # resize the window for the new email width
        self.pop_up_window.resize(self.pop_up_window.minimumSizeHint())

        # we want the popup to be aligned with the bottom right hand corner of the button
        # Do after showing so that the geometry is correct
        btn_width = self.width()
        pop_up_width = self.pop_up_window.width()
        right_shift = btn_width - pop_up_width - 4
        pop_up_pos = self.mapToGlobal(QtCore.QPoint(right_shift, self.height() + 4))
        self.pop_up_window.move(pop_up_pos)


class PopUpWindow(QtWidgets.QDialog, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        QtWidgets.QDialog.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        self.setProperty("popup", True)  # used for styling
        self.setWindowFlags(QtCore.Qt.Popup)
        plugin.get_user_interface().set_style_sheet(self)

        self.create_widgets()
        self.create_layout()

    def get_plugin(self):
        return self._plugin

    def create_widgets(self):
        user_interface = self.get_plugin().get_user_interface()

        self.email_label = QtWidgets.QLabel("")
        self.email_label.setProperty("user-name-header", True)
        self.email_label.setAlignment(QtCore.Qt.AlignLeft)
        self.email_label.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)

        self.user_badge = QtWidgets.QLabel()
        self.user_badge.setPixmap(qt_constants.USER_ICON_OFF_WHITE.pixmap(48))
        self.user_badge.setAlignment(QtCore.Qt.AlignLeft)
        self.user_badge.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)

        self.user_badge_layout = QtWidgets.QHBoxLayout()
        self.user_badge_layout.setAlignment(QtCore.Qt.AlignLeft)
        self.user_badge_layout.addWidget(self.user_badge)
        self.user_badge_layout.addWidget(self.email_label)
        self.user_badge_layout.setContentsMargins(5, 2, 8, 2)

        self.menu = QtWidgets.QMenu()
        self.menu.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        self.menu.setProperty('inPopup', True)
        self.menu.setWindowFlags(self.menu.windowFlags() | QtCore.Qt.FramelessWindowHint)

        sign_out_action = QtWidgets.QAction(qt_constants.USER_ICON_OFF_WHITE, u"Sign-Out", self)
        sign_out_action.triggered.connect(self.sign_out)
        self.menu.addAction(sign_out_action)

    def about_to_hide(self):
        # There is a weird space to the right of the menu which will make the menu disappear if clicked by the user
        # This disables that
        self.menu.show()

    def create_layout(self):
        layout = QtWidgets.QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setAlignment(QtCore.Qt.AlignLeft)
        layout.addLayout(self.user_badge_layout)
        layout.addWidget(self.menu)

    def sign_out(self):
        plugin = self.get_plugin()
        api_client = plugin.get_api_client()
        user_interface = plugin.get_user_interface()
        main_window = user_interface.get_main_window()

        # sign out the current user
        api_client.sign_out()

        # close the popup
        self.close()

        # reset the main window
        main_window.reset()
