__all__ = 'ListRadioInput'

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

from .input_widget import InputWidget
from gm_qt_plugin import qt_constants, qt_utils
from gm_qt_plugin.widgets.input_widgets.spin_box import SpinBox
from gm_qt_plugin.widgets.input_widgets.check_box import CheckBox
from gm_qt_plugin.widgets.elided_label import ElidedLabel
from gm_meta_plugin.inference_source import InferenceSource


class ListRadioInput(QtWidgets.QWidget, InputWidget):

    def __init__(self, plugin, list_attribute, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)

        self._items = []

        self._list_attribute = list_attribute

        """ Create Widgets """
        self.internal_widget = QtWidgets.QWidget()

        self._scroll_area = QtWidgets.QScrollArea()
        self._scroll_area.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        self._scroll_area.setWidgetResizable(True)
        self._scroll_area.setProperty("background-level-2", True)
        self._scroll_area.viewport().setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        self._scroll_area.setWidget(self.internal_widget)

        """ Create Layout """
        self._items_layout = QtWidgets.QGridLayout(self.internal_widget)
        #self._items_layout.setContentsMargins(0, 0, 0, 0)
        self._items_layout.setAlignment(QtCore.Qt.AlignTop)
        self._items_layout.setColumnStretch(0, 1)

        self.layout = QtWidgets.QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self._scroll_area)

    def update_items(self):

        if self._list_attribute.get_items_inference_source() != InferenceSource.get_application_inference_source():
            raise NotImplementedError()

        application_attribute_source = self.get_plugin().get_application_attribute_source()
        old_items = self._items

        res = application_attribute_source.get_enum_items(None, None, self._list_attribute.get_key())
        try:
            items, default_item_index = res
        except TypeError as e:
            print("res:", res)
            print("key:", self._list_attribute.get_key())
            raise e

        value = application_attribute_source.get_attribute_value(None, None, self._list_attribute.get_key())

        # If the options have changed then update the widgets
        old_item_keys = list(map(lambda x: x.get_key(), old_items))
        item_keys = list(map(lambda x: x.get_key(), items))

        if old_item_keys != item_keys:

            self._items = items

            qt_utils.delete_layout_items(self._items_layout)

            for row, item in enumerate(self._items):
                label = QtWidgets.QLabel(item.get_display_name())
                check_box = CheckBox(self.get_plugin())
                check_box.set_value(item.get_key() in value)

                self._items_layout.addWidget(label, row, 0)
                self._items_layout.addWidget(check_box, row, 1)

            self._scroll_area.setFixedHeight((min(5, len(items)) * 26) + (self._items_layout.contentsMargins().top() * 2))

            # flash to indicate to the user that the items have changed
            user_interface = self.get_plugin().get_user_interface()
            colors = user_interface.get_theme()
            original_color = QtGui.QColor(colors.FOREGROUND_COLOR)
            flash_color = QtGui.QColor(colors.BRAND_COLOR)
            qt_utils.flash_widget_color(self, original_color, flash_color)

    def get_value(self):

        value = []
        for row in range(self._items_layout.rowCount()):
            layout_item = self._items_layout.itemAtPosition(row, 1)

            if isinstance(layout_item, QtWidgets.QWidgetItem):
                combo_box = layout_item.widget()

                if combo_box.get_value():
                    value.append(self._items[row].get_key())

        return value

    def set_value(self, value):

        for row in range(self._items_layout.rowCount()):
            layout_item = self._items_layout.itemAtPosition(row, 1)

            if isinstance(layout_item, QtWidgets.QWidgetItem):
                item = self._items[row]
                combo_box = layout_item.widget()
                combo_box.set_value(item.get_key() in value)

