__all__ = 'PluginsInput'

from .input_widget import InputWidget
from PySide2 import QtWidgets
from PySide2 import QtCore
from gm_qt_plugin import qt_constants
from gm_qt_plugin.widgets.input_widgets.spin_box import SpinBox
from gm_qt_plugin.widgets.input_widgets.check_box import CheckBox
from gm_qt_plugin.widgets.elided_label import ElidedLabel


class PluginsInput(QtWidgets.QWidget, InputWidget):

    def __init__(self, plugin, job_preset, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)

        self._job_preset = job_preset
        self._app = None
        self._version = None

        """ Create Widgets """
        self.group_box = QtWidgets.QGroupBox()
        self.group_box.setProperty("background-level-2", True)

        self.list = QtWidgets.QListWidget()
        self.list.setProperty("background-level-2", True)
        self.list.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.list.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.list.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        self.list.setAlternatingRowColors(True)

        self._warning_label = QtWidgets.QWidget()
        self._warning_label_text = ElidedLabel("Warning: frame ranges overlap. You may be charged for overlapping " 
                                               "frames but will receive only one output frame for each conflict.")
        self._warning_label_text.setColor('grey')

        self._warning_label_icon = QtWidgets.QLabel()
        self._warning_label_icon.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        self._warning_label_icon.setPixmap(qt_constants.WARNING_ICON_OFF_WHITE.pixmap(24))
        self._warning_label.setVisible(False)  # starts invisible

        """ Create Layout """
        self.vlayout = QtWidgets.QVBoxLayout()
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.vlayout.setAlignment(QtCore.Qt.AlignTop)

        self.hlayout = QtWidgets.QHBoxLayout(self.group_box)
        self.hlayout.setContentsMargins(0, 0, 0, 0)
        self.hlayout.addWidget(self.list)
        self.hlayout.addLayout(self.vlayout)

        self._warning_label_layout = QtWidgets.QHBoxLayout(self._warning_label)
        self._warning_label_layout.setContentsMargins(0, 0, 0, 0)
        self._warning_label_layout.addWidget(self._warning_label_icon)
        self._warning_label_layout.addWidget(self._warning_label_text)

        self.layout = QtWidgets.QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self.group_box)
        self.layout.addWidget(self._warning_label)

        """ Create Connections """
        self.input_value_changed.connect(self._update_warning_label)

    @QtCore.Slot()
    def handle_job_preset_attribute_value_changed(self, job_attribute_id: str):
        print("handle_job_preset_attribute_value_changed:", job_attribute_id)

        job_preset_attribute = self._job_preset

    def _update_warning_label(self):
        pass

    def _update_list_height(self):
        item_count = self.list.count()

        base_height = -35
        item_height = 40

        if item_count <= 1:
            height = base_height + (item_height * 2)
        elif item_count == 2:
            height = base_height + (item_height * 3)
        elif item_count == 3:
            height = base_height + (item_height * 4)
        elif item_count == 4:
            height = base_height + (item_height * 5)
        else:
            height = base_height + (item_height * 6)

        self.list.setFixedHeight(height)

    def minimumSizeHint(self):
        return QtCore.QSize(0, 0)

    def get_value(self):
        return self.to_string()

    def set_value(self, value):
        pass

    def to_string(self):
        return 'string'
