__all__ = 'SpinBox'

from .input_widget import InputWidget

from PySide2 import QtCore
from PySide2 import QtWidgets


class SpinBox(QtWidgets.QSpinBox, InputWidget):

    def __init__(self, plugin, parent=None):
        QtWidgets.QSpinBox.__init__(self, parent=parent)
        InputWidget.__init__(self, plugin)

        self.valueChanged.connect(self._catch_value_change)

        # set strong focus so input does not gain focus from mouse wheel events
        self.setFocusPolicy(QtCore.Qt.StrongFocus)

    def get_value(self):
        return self.value()

    def set_value(self, value):
        if isinstance(value, str):
            value = int(value)

        self.setValue(value)

    def wheelEvent(self, event):
        if self.hasFocus():
            return QtWidgets.QSpinBox.wheelEvent(self, event)
        else:
            event.ignore()

