__all__ = 'MainUI'

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

from gm_qt_plugin import qt_constants, qt_utils
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor

from gm_qt_plugin.widgets.header import Header
from gm_qt_plugin.widgets.main_ui_switcher import MainUISwitcher


class MainUI(QtWidgets.QWidget, QtPluginAccessor):
    MIN_WINDOW_WIDTH = 800
    MIN_WINDOW_HEIGHT = 775

    MAX_WINDOW_WIDTH = qt_constants.DEFAULT_MAX_SIZE
    MAX_WINDOW_HEIGHT = qt_constants.DEFAULT_MAX_SIZE

    INITIAL_WINDOW_WIDTH = 1314
    INITIAL_WINDOW_HEIGHT = 720
    INITIAL_LOGGING_HEIGHT = 200

    def __init__(self, plugin, parent=None):

        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        """ Create Widgets """
        # header
        self.header = Header(plugin)

        # min ui switcher
        self.main_ui_switcher = MainUISwitcher(plugin)

        # logging container
        self.logging_container = plugin.get_logging_coordinator().get_log_history_container()

        """ Create Layouts """
        self.layout = QtWidgets.QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(0)
        self.layout.setAlignment(QtCore.Qt.AlignTop)

        self.scroll_area = QtWidgets.QScrollArea()
        self.scroll_area.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scroll_area.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scroll_area.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setWidget(self.main_ui_switcher)
        self.scroll_area.setProperty("background-off-white", True)

        self._splitter = QtWidgets.QSplitter(QtCore.Qt.Vertical)
        self._splitter.addWidget(self.scroll_area)
        self._splitter.addWidget(self.logging_container)
        self._splitter.setSizes([700, 0])
        self.logging_container.hide_header_controls()

        self._splitter.setHandleWidth(25)
        self._splitter.splitterMoved.connect(self.splitter_moved_slot)
        self._splitter.handle(1).setLayout(self.logging_container.get_header_layout())

        self.layout.addWidget(self._splitter)

        self.h_layout = QtWidgets.QHBoxLayout()
        self.h_layout.setContentsMargins(0, 0, 0, 0)
        self.h_layout.setSpacing(0)
        self.h_layout.addLayout(self.layout)

        self.v_layout = QtWidgets.QVBoxLayout(self)
        self.v_layout.setContentsMargins(0, 0, 0, 0)
        self.v_layout.setSpacing(0)
        self.v_layout.addWidget(self.header)
        self.v_layout.addLayout(self.h_layout)

    def splitter_moved_slot(self, pos, index):
        is_logging_container_collapsed = self._splitter.sizes()[1] == 0

        # Show / hide the logging container header controls based on the widgets collapsed state
        if is_logging_container_collapsed:
            self.logging_container.hide_header_controls()
        else:
            self.logging_container.show_header_controls()

    def get_header(self):
        return self.header

    def get_main_ui_switcher(self):
        return self.main_ui_switcher
