__all__ = 'MainUISwitcher'

from PySide2 import QtCore
from PySide2 import QtWidgets
from PySide2 import QtGui

from gm_qt_plugin import qt_constants
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor

from gm_qt_plugin.widgets.import_existing_remote_project_widget import ImportExistingRemoteProjectWidget
from gm_qt_plugin.widgets.pack_current_scene_widget import PackCurrentSceneWidget
from gm_qt_plugin.widgets.upload_current_scene_widget import UploadCurrentSceneWidget
from gm_qt_plugin.widgets.upload_packed_project_widget import UploadPackedProjectWidget
from gm_qt_plugin.widgets.submission_view_widget import SubmissionViewWidget
from gm_qt_plugin.widgets.resizable_stacked_widget import ResizableStackedWidget
from gm_qt_plugin.widgets.top_banner import TopBanner

class MainUISwitcher(QtWidgets.QWidget, QtPluginAccessor):

    def __init__(self, plugin, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        self.job_preset_container_view_index = None
        self.existing_remote_projects_view_index = None
        self.remote_projects_container_view_index = None
        self.pack_current_scene_view_index = None
        self.upload_current_scene_view_index = None
        self.upload_packed_project_view_index = None
        self.submission_view_index = None

        """ Create Widgets """
        # top banner
        self.top_banner = TopBanner(plugin)

        self._stacked_widget = ResizableStackedWidget()
        self._stacked_widget.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

        """ Create Layouts """
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

        self.top_bar_layout = QtWidgets.QHBoxLayout()
        self.top_bar_layout.addWidget(self.top_banner)
        self.top_bar_layout.setAlignment(QtCore.Qt.AlignHCenter)

        self._layout = QtWidgets.QVBoxLayout(self)
        self._layout.setAlignment(QtCore.Qt.AlignTop)
        self._layout.setContentsMargins(qt_constants.MARGIN_SIZE,
                                        qt_constants.MARGIN_SIZE,
                                        qt_constants.MARGIN_SIZE,
                                        qt_constants.MARGIN_SIZE)
        self._layout.addLayout(self.top_bar_layout)
        self._layout.addSpacing(10)
        self._layout.addWidget(self._stacked_widget)

        #self.display_job_preset_container()
        #self.display_import_existing_remote_projects()
        #self.display_remote_project_container()
        #self.display_pack_current_scene_view()
        #self.display_upload_current_scene_view()
        #self.display_upload_packed_project_view()
        self.display_submission_view()

    def get_submission_view_widget(self):
        if self.submission_view_index is not None:
            return self._stacked_widget.widget(self.submission_view_index)

    def get_current_widget(self):
        return self._stacked_widget.currentWidget()

    def display_job_preset_container(self):
        return

        # hide job preset container for v1
        """
        if self.job_preset_container_view_index is None:
            widget = self.get_plugin().get_preferences_container().get_job_preset_container()
            index = self._stacked_widget.addWidget(widget)
            self.job_preset_container_view_index = index

        self._stacked_widget.setCurrentIndex(self.job_preset_container_view_index)

        # redraw the current focused job preset in the job preset container
        job_preset_container_widget = self._stacked_widget.currentWidget()
        current_item = job_preset_container_widget.job_preset_to_tree_item(job_preset_container_widget.get_focused_item())
        job_preset_container_widget.current_item_changed(current_item, current_item)
        """

    def display_import_existing_remote_projects(self):
        return
        """
        if self.existing_remote_projects_view_index is None:
            widget = ImportExistingRemoteProjectWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.existing_remote_projects_view_index = index

        self._stacked_widget.setCurrentIndex(self.existing_remote_projects_view_index)
        """

    def display_remote_project_container(self):
        return

        # hide remote project container for v1
        """
        if self.remote_projects_container_view_index is None:
            widget = self.get_plugin().get_remote_project_container()
            index = self._stacked_widget.addWidget(widget)
            self.remote_projects_container_view_index = index

        self._stacked_widget.setCurrentIndex(self.remote_projects_container_view_index)
        """

    def display_pack_current_scene_view(self):
        return

        """
        if self.pack_current_scene_view_index is None:
            widget = PackCurrentSceneWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.pack_current_scene_view_index = index

        self._stacked_widget.setCurrentIndex(self.pack_current_scene_view_index)
        """

    def display_upload_current_scene_view(self):
        return

        """
        if self.upload_current_scene_view_index is None:
            widget = UploadCurrentSceneWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.upload_current_scene_view_index = index

        self._stacked_widget.setCurrentIndex(self.upload_current_scene_view_index)
        """

    def display_upload_packed_project_view(self):
        return

        """
        if self.upload_packed_project_view_index is None:
            widget = UploadPackedProjectWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.upload_packed_project_view_index = index

        self._stacked_widget.setCurrentIndex(self.upload_packed_project_view_index)
        """

    def display_submission_view(self):
        if self.submission_view_index is None:
            widget = SubmissionViewWidget(self.get_plugin())
            index = self._stacked_widget.addWidget(widget)
            self.submission_view_index = index

        self._stacked_widget.setCurrentIndex(self.submission_view_index)
