__all__ = 'PixmapLabel'

from PySide2 import QtWidgets, QtGui, QtCore
from gm_qt_plugin.widgets.clickable_qlabel import ClickableQLabel


class PixmapLabel(ClickableQLabel):

    def __init__(self, parent=None):
        ClickableQLabel.__init__(self, parent=parent)

        self._text = None
        self._pixmap = None

        # scale content by default
        self.setScaledContents(True)

        # QImage to cache the pixmap()
        # to avoid constructing a new QImage on every scale operation
        self.cachedImage = None

        # used to cache the last scaled pixmap
        # to avoid calling scale again when the size is still at the same
        self.scaledPixmap = None

        # key for the currently cached QImage and QPixmap
        # used to make sure the label was not set to another QPixmap
        self.cacheKey = None

    def text(self):
        return self._text

    def setText(self, text):
        self._text = text

    def pixmap(self):
        return self._pixmap

    def setPixmap(self, pixmap):
        self._pixmap = pixmap

    def paintEvent(self, event):

        # if self is assigned a pixmap
        if self.pixmap():
            style= self.style()
            painter = QtGui.QPainter(self)

            self.drawFrame(painter)
            cr = self.contentsRect()
            cr.adjust(self.margin(), self.margin(), -self.margin(), -self.margin())

            align = QtWidgets.QStyle.visualAlignment(self.layoutDirection(), self.alignment())
            if self.hasScaledContents():  # if scaling is enabled
                scaledSize = cr.size() * self.devicePixelRatioF()

                # if scaledPixmap is invalid
                if self.scaledPixmap is None or self.scaledPixmap.size() != scaledSize or \
                    self.pixmap().cacheKey() != self.cacheKey:

                    # if cachedImage is also invalid
                    if self.pixmap().cacheKey() != self.cacheKey:

                        # reconstruct cachedImage
                        self.cachedImage = self.pixmap().toImage()

                    scaledImage = self.cachedImage.scaled(scaledSize,
                                                          QtCore.Qt.KeepAspectRatioByExpanding,
                                                          QtCore.Qt.SmoothTransformation)

                    self.scaledPixmap = QtGui.QPixmap.fromImage(scaledImage)
                    self.scaledPixmap.setDevicePixelRatio(self.devicePixelRatioF())

                pix = self.scaledPixmap
            else:  # no scaling, Just use pixmap()
                pix = self.pixmap()

            opt = QtWidgets.QStyleOption()
            opt.initFrom(self)

            if not self.isEnabled():
                pix = style.generatedIconPixmap(QtGui.QIcon.Disabled, pix, opt)

            style.drawItemPixmap(painter, cr, align, pix)
            style.drawItemText(painter, cr, QtWidgets.QStyle.visualAlignment(self.layoutDirection(), QtCore.Qt.AlignVCenter | QtCore.Qt.AlignLeft),
                               self.palette(), True, self.text())
        else:  # otherwise (if the label is not assigned to a pixmap)
            # call base paintEvent
            QtWidgets.QLabel.paintEvent(self, event)
