__all__ = "LeftBanner"

from PySide2 import QtCore
from PySide2 import QtWidgets
from PySide2 import QtGui

from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor
from gm_qt_plugin.widgets.pixmap_label import PixmapLabel
from gm_qt_plugin import qt_constants, qt_utils


class TopBanner(PixmapLabel, QtPluginAccessor):
    BANNER_HEIGHT = 74

    SMALL_BANNER_IMAGE = QtGui.QPixmap(qt_constants.DEFAULT_TOP_BANNER_SHORT_IMAGE)
    WIDE_BANNER_IMAGE = QtGui.QPixmap(qt_constants.DEFAULT_TOP_BANNER_WIDE_IMAGE)
    BREAK_POINT = 800

    def __init__(self, plugin, parent=None):
        PixmapLabel.__init__(self, parent=parent)
        QtPluginAccessor.__init__(self, plugin)

        self.setText(u"    60 machines not enough?"
                     "\n    Please contact us if you need more!"
                     "\n    Let's render...")

        self.setPixmap(TopBanner.SMALL_BANNER_IMAGE)
        self.setFixedHeight(TopBanner.BANNER_HEIGHT)
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        self.setAlignment(QtCore.Qt.AlignCenter)
        self.setCursor(QtCore.Qt.PointingHandCursor)
        pal = self.palette()
        pal.setColor(self.foregroundRole(), QtGui.QColor(255, 0, 0))
        self.setPalette(pal)
        self.setStyleSheet("color: white;")  # used to style the font color of the top banner

        banner_font = QtGui.QFont("Open Sans")
        banner_font.setBold(True)
        banner_font.setPixelSize(30)
        self.setFont(banner_font)
        self.clicked.connect(self.handle_clicked_signal)

    def handle_clicked_signal(self):
        qt_utils.open_url(self.get_plugin().get_user_interface().get_left_banner_link())

    def resizeEvent(self, event):
        width = event.size().width()

        if width > TopBanner.BREAK_POINT:
            self.setPixmap(TopBanner.WIDE_BANNER_IMAGE)
        else:
            self.setPixmap(TopBanner.SMALL_BANNER_IMAGE)

        QtWidgets.QWidget.resizeEvent(self, event)
