__all__ = 'APIClient'

import sys
import typing

from gm_qt_plugin import QtAPIClient
from gm_meta_plugin.gridmarkets import constants as api_constants
from submit2gm import constants

if typing.TYPE_CHECKING:
    from gm_meta_plugin.factory_collection import FactoryCollection
    from gm_meta_plugin.gridmarkets.api_schema import APISchema
    from gm_meta_plugin.attribute_types import EnumAttributeType


class APIClient(QtAPIClient):

    def __init__(self, plugin, logging_coordinator, factory_collection: 'FactoryCollection'):
        QtAPIClient.__init__(self, plugin, logging_coordinator, factory_collection)
        self._modified_api_cache = False

    def sign_in(self, user, skip_validation = False):
        QtAPIClient.sign_in(self, user, skip_validation=skip_validation)

    def get_api_schema(self, factory_collection: 'FactoryCollection' = None, ignore_cache: bool = False) -> 'APISchema':
        api_schema = QtAPIClient.get_api_schema(self, factory_collection=factory_collection, ignore_cache=ignore_cache)

        if constants.JOB_DEFINITION_ID_FLAG in sys.argv and (not self._modified_api_cache or ignore_cache):
            project_product_attribute: 'EnumAttributeType' = api_schema.get_project_definition_schema().get_project_attribute_with_id(
                api_constants.PROJECT_ATTRIBUTE_IDS.PRODUCT).get_attribute()

            items = project_product_attribute.get_items()
            job_definition_id = sys.argv[sys.argv.index(constants.JOB_DEFINITION_ID_FLAG) + 1]

            job_definition = api_schema.get_job_definition_with_id(job_definition_id)
            supported_applications = api_schema.get_supported_applications(job_definition)

            modified_items = []
            for item in items:
                if item.get_key() in supported_applications:
                    modified_items.append(item)

            project_product_attribute._items = modified_items

            self._modified_api_cache = True

        return api_schema
