__all__ = 'Submit2gmAttributeSource'

import typing

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from submit2gm import constants


if typing.TYPE_CHECKING:
    from .submit2gm_plugin import Submit2gmPlugin
    from submit2gm.submit2gm_dyn.blender.blender_attribute_source import BlenderAttributeSource
    from submit2gm.submit2gm_dyn.c4d.c4d_attribute_source import C4DAttributeSource
    from submit2gm.submit2gm_dyn.max.max_attribute_source import MaxAttributeSource
    from submit2gm.submit2gm_dyn.maya.maya_attribute_source import MayaAttributeSource
    from submit2gm.submit2gm_dyn.nuke.nuke_attribute_source import NukeAttributeSource


class Submit2gmAttributeSource(gm_meta_plugin.ApplicationAttributeSource):

    def __init__(self, plugin: 'Submit2gmPlugin'):
        gm_meta_plugin.ApplicationAttributeSource.__init__(self, plugin)

        self._blender_attribute_source: typing.Optional['BlenderAttributeSource'] = None
        self._c4d_attribute_source: typing.Optional['C4DAttributeSource'] = None
        self._max_attribute_source: typing.Optional['MaxAttributeSource'] = None
        self._maya_attribute_source: typing.Optional['MayaAttributeSource'] = None
        self._nuke_attribute_source: typing.Optional['NukeAttributeSource'] = None

    def get_blender_attribute_source(self) -> 'BlenderAttributeSource':
        if self._blender_attribute_source is None:
            from submit2gm.submit2gm_dyn.blender.blender_attribute_source import BlenderAttributeSource
            self._blender_attribute_source = BlenderAttributeSource(self.get_plugin())
        return self._blender_attribute_source

    def get_c4d_attribute_source(self) -> 'C4DAttributeSource':
        if self._c4d_attribute_source is None:
            from submit2gm.submit2gm_dyn.c4d.c4d_attribute_source import C4DAttributeSource
            self._c4d_attribute_source = C4DAttributeSource(self.get_plugin())
        return self._c4d_attribute_source

    def get_max_attribute_source(self) -> 'MaxAttributeSource':
        if self._max_attribute_source is None:
            from submit2gm.submit2gm_dyn.max.max_attribute_source import MaxAttributeSource
            self._max_attribute_source = MaxAttributeSource(self.get_plugin())
        return self._max_attribute_source

    def get_maya_attribute_source(self) -> 'MayaAttributeSource':
        if self._maya_attribute_source is None:
            from submit2gm.submit2gm_dyn.maya.maya_attribute_source import MayaAttributeSource
            self._maya_attribute_source = MayaAttributeSource(self.get_plugin())
        return self._maya_attribute_source

    def get_nuke_attribute_source(self) -> 'NukeAttributeSource':
        if self._nuke_attribute_source is None:
            from .submit2gm_dyn.nuke.nuke_attribute_source import NukeAttributeSource
            self._nuke_attribute_source = NukeAttributeSource(self.get_plugin())
        return self._nuke_attribute_source

    def _get_attribute_source(self, app, app_version, key) -> 'gm_meta_plugin.ApplicationAttributeSource':

        cps = self.get_plugin().get_active_command_port_server()
        if app is None:
            app = cps.get_product()

        if app == api_constants.PRODUCTS.BLENDER:
            return self.get_blender_attribute_source()

        if app == api_constants.PRODUCTS.C4D:
            return self.get_c4d_attribute_source()

        if app == api_constants.PRODUCTS.MAX:
            return self.get_max_attribute_source()

        if app == api_constants.PRODUCTS.MAYA:
            return self.get_maya_attribute_source()

        if app == api_constants.PRODUCTS.NUKE:
            return self.get_nuke_attribute_source()

        raise ValueError("{} does not support product '{}'".format(constants.PROGRAM_NAME, str(app)))

    def get_enum_items(self, app, app_version, key):

        cps = self.get_plugin().get_active_command_port_server()
        if app is None:
            app = cps.get_product()

        return self._get_attribute_source(app, app_version, key).get_enum_items(app, app_version, key)

    def get_attribute_value(self, app, app_version, key):

        cps = self.get_plugin().get_active_command_port_server()
        if app is None:
            app = cps.get_product()

        return self._get_attribute_source(app, app_version, key).get_attribute_value(app, app_version, key)
