__all__ = 'BlenderAttributeSource'

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_meta_plugin.errors import ApplicationAttributeNotFound
from gm_meta_plugin.attribute_types import EnumItem
from submit2gm import constants
from . import blender_constants, blender_cpc


class BlenderAttributeSource(gm_meta_plugin.ApplicationAttributeSource):

    def get_enum_items(self, app: str, app_version: str, key: str):
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()

        if key == api_constants.API_KEYS.APP_VERSION:
            items = []
            product_versions = self.get_plugin().get_api_client().get_product_versions(app)
            for product_version in product_versions:
                items.append(EnumItem(product_version, product_version, u""))
            return items, self._get_enum_index(app, app_version, key, items)

        if key == api_constants.API_KEYS.MACHINE_TYPE:
            return plugin.get_api_client().get_machines_options(api_constants.PRODUCTS.BLENDER,
                                                                api_constants.OPERATION_TYPES.RENDER,
                                                                use_gpu_default=True)

        raise ApplicationAttributeNotFound(app, app_version, key)

    def get_attribute_value(self, app: str, app_version: str, key: str):
        cps = self.get_plugin().get_active_command_port_server()

        current_renderer = blender_cpc.get_render_engine(cps)

        if app != api_constants.PRODUCTS.BLENDER:
            raise ValueError(str(app))

        if key == api_constants.API_KEYS.IS_SAVED:
            return blender_cpc.is_saved(cps)

        if key == api_constants.API_KEYS.PROJECT_NAME:
            blend_file_name = blender_cpc.get_scene_name(cps)

            if blend_file_name.endswith(blender_constants.BLEND_FILE_EXTENSION):
                blend_file_name = blend_file_name[:-len(blender_constants.BLEND_FILE_EXTENSION)]

            if not blend_file_name:
                return "Untitled"

            return blend_file_name

        if key == api_constants.API_KEYS.SUBMISSION_NAME:
            blend_file_name = blender_cpc.get_scene_name(cps)

            if not blend_file_name:
                return "Untitled"

            if not blend_file_name.endswith(blender_constants.BLEND_FILE_EXTENSION):
                blend_file_name = blend_file_name + blender_constants.BLEND_FILE_EXTENSION

            return blend_file_name

        if key == api_constants.API_KEYS.JOB_NAME:
            job_name_parts = []

            active_camera_name = blender_cpc.get_active_camera_name(cps)
            if active_camera_name:
                job_name_parts.append(active_camera_name)

            if current_renderer:
                job_name_parts.append(current_renderer)

            return ' '.join(list(map(lambda x: "[{}]".format(x), job_name_parts)))

        if key == api_constants.API_KEYS.APP:
            return api_constants.PRODUCTS.BLENDER

        if key == api_constants.API_KEYS.APP_VERSION:
            blender_version = blender_cpc.get_version_tuple(cps)
            version_string = str(blender_version[0]) + '.' + str(blender_version[1]) + '.' + str(blender_version[2])
            return self.get_plugin().get_api_client().get_closest_matching_product_version(
                api_constants.PRODUCTS.BLENDER,
                version_string)

        if key == api_constants.API_KEYS.SCENE_FILE_PATH_UNUSED:
            return blender_cpc.get_file_path(cps)

        if key == api_constants.API_KEYS.RENDER_ENGINE:
            return current_renderer

        if key == api_constants.API_KEYS.OUTPUT_PREFIX:
            return blender_cpc.get_output_path(cps)

        if key == api_constants.API_KEYS.OUTPUT_FORMAT:
            return blender_cpc.get_output_format(cps)

        elif key == api_constants.API_KEYS.FRAMES:
            return blender_cpc.get_frame_range_str(cps)

        raise ApplicationAttributeNotFound(app, app_version, key)
