from . import blender_constants

import typing

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import CommandPortServer


def is_saved(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
is_saved = bpy.data.is_saved
""", return_variable_name='is_saved')


def is_dirty(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
is_dirty = bpy.data.is_dirty
""", return_variable_name='is_dirty')


def save(cps: 'CommandPortServer'):
    return cps.eval("""
bpy.ops.wm.save_as_mainfile(check_existing=False)
""")


def get_file_path(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
blend_file_path = bpy.context.blend_data.filepath.replace('\\\\', '/')
""", return_variable_name='blend_file_path')


def get_scene_name(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
blend_file_name = bpy.path.basename(bpy.context.blend_data.filepath) 
""", return_variable_name='blend_file_name')


def get_active_camera_name(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
active_camera = bpy.context.scene.camera
active_camera_name = ''

if active_camera is not None and active_camera.type == 'CAMERA':
    active_camera_name = active_camera.name
""", return_variable_name='active_camera_name')


def get_version_tuple(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
blender_version = bpy.app.version
""", return_variable_name='blender_version')


def get_render_engine(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
render_engine =  bpy.context.engine
""", return_variable_name='render_engine')


def get_output_path(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
output_path = bpy.context.scene.render.filepath
""", return_variable_name='output_path')


def get_output_format(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
output_format = bpy.context.scene.render.image_settings.file_format
""", return_variable_name='output_format')


def get_frame_range_str(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
frame_range_str = str(bpy.context.scene.frame_start) + ' ' + str(bpy.context.scene.frame_end) + ' ' + str(bpy.context.scene.frame_step)
""", return_variable_name='frame_range_str')


def get_local_download_path(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
local_download_path = ""
import pathlib
local_download_path = pathlib.Path(bpy.path.abspath(bpy.context.scene.render.filepath)).resolve().as_posix()
""", return_variable_name='local_download_path')


def save_main_file(cps: 'CommandPortServer', target: str):
    return cps.eval("""
bpy.ops.wm.save_as_mainfile(copy=True, filepath=fr'{}', relative_remap=True, compress=True)
""".format(target))


def pack_scene(cps: 'CommandPortServer', target_file: str, output_dir: str):
    return cps.eval("""
import pathlib
from blender_asset_tracer import pack
import logging
log = logging.getLogger(__name__)
log.info("Starting pack operation....")

blend_file_path = pathlib.Path(fr'{}')
project_root_path = blend_file_path.parent
target_dir_path = pathlib.Path(fr'{}')

log.info("blend_file_path: %s" % str(blend_file_path))
log.info("project_root_path: %s" % str(project_root_path))
log.info("target_dir_path: %s" % str(target_dir_path))

progress_cb = None # BatProgressCallback(log)

# create packer
with pack.Packer(blend_file_path,
                 project_root_path,
                 target_dir_path,
                 noop=False,  # no-op mode, only shows what will be done
                 compress=False,
                 relative_only=False
                 ) as packer:

    log.info("Created packer")

    if progress_cb:
        log.info("Setting packer progress callback...")
        packer._progress_cb = progress_cb

    # plan the packing operation (must be called before execute)
    log.info("Plan packing operation...")
    packer.strategise()

    # attempt to pack the project
    try:
        log.info("Execute packing operation...")
        packer.execute()
    except pack.transfer.FileTransferError as ex:
        log.warning(str(len(ex.files_remaining)) + " files couldn't be copied, starting with " +
                    str(ex.files_remaining[0]))
        raise SystemExit(1)
    except Exception as e:
        log.error(utils.get_exception_with_traceback(e))
        raise e
    finally:
        log.info("Exiting packing operation...")
""".format(target_file, output_dir))
