import pathlib

from gm_meta_plugin.file_packer import FilePacker
from gm_meta_plugin.packed_project import PackedProject
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_meta_plugin import utils
from . import blender_cpc


class BlenderFilePacker(FilePacker):
    BAT_PACK_INFO_FILE_NAME = "pack-info.txt"

    def __init__(self, plugin):
        FilePacker.__init__(self, plugin)

    def pack(self, target_file: pathlib.Path, output_dir: pathlib.Path) -> PackedProject:
        plugin = self.get_plugin()
        logging_coordinator = plugin.get_logging_coordinator()
        log = logging_coordinator.get_logger(__name__)

        cps = self.get_plugin().get_active_command_port_server()

        # pack the .blend file to the pack directory
        blender_cpc.pack_scene(cps, str(target_file), str(output_dir))

        # delete pack-info.txt if it exists
        pack_info_file = output_dir / self.BAT_PACK_INFO_FILE_NAME
        if pack_info_file.is_file():
            log.info("Removing '%s'..." % str(pack_info_file))
            pack_info_file.unlink()

        project_name = output_dir.stem

        files = utils.get_files_in_directory(output_dir)
        attributes = {
            api_constants.API_KEYS.PATH: str(output_dir / target_file.name),
        }

        log.info("Finished packing project \"" + str(target_file) + "\" to \"" + str(output_dir) + "\"")
        return PackedProject(project_name,
                             output_dir,
                             files,
                             attributes)
