import tempfile
import pathlib

from gm_meta_plugin.packed_project import PackedProject
from gm_meta_plugin.scene_exporter import SceneExporter
from .blender_file_packer import BlenderFilePacker
from . import blender_constants, blender_cpc


class BlenderSceneExporter(SceneExporter):

    def __init__(self, plugin):
        SceneExporter.__init__(self, plugin)

    def _save_scene_to_file(self, target: str):
        log = self.get_plugin().get_logging_coordinator().get_logger(__name__)
        log.info("Saving scene to temporary file: " + target)
        cps = self.get_plugin().get_active_command_port_server()
        blender_cpc.save_main_file(cps, target)

    def export(self, output_dir: pathlib.Path) -> PackedProject:
        # save a copy of blender scene to a temporary .blend file
        tmp_dir = tempfile.TemporaryDirectory(prefix='gm-temp-dir-')

        cps = self.get_plugin().get_active_command_port_server()

        # Get the file name for the scene
        blend_file_name = blender_cpc.get_scene_name(cps)

        if not blend_file_name:
            blend_file_name = blender_constants.DEFAULT_PROJECT_NAME

        if not blend_file_name.endswith('.blend'):
            blend_file_name = blend_file_name + blender_constants.BLEND_FILE_EXTENSION

        # Get the temporary path to save a copy of the scene to
        blend_file_path = pathlib.Path(tmp_dir.name) / blend_file_name

        self._save_scene_to_file(str(blend_file_path))

        packed_project = BlenderFilePacker(self.get_plugin()).pack(blend_file_path, output_dir)

        # delete temporary pre-packed .blend file
        tmp_dir.cleanup()

        return packed_project
