from . import c4d_constants

import typing

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import CommandPortServer





def _get_take(take_name: str):
    return """
take_name = "{}"
document = c4d.documents.GetActiveDocument()
take_data = document.GetTakeData()

take = take_data.GetMainTake()

# Get the take with the specified name
takes = [take]
while takes:
    next_take = takes.pop(0)
    
    if next_take.GetName() == take_name:
        take = next_take
        break
    else:
        takes.extend(take.GetChildren())
""".format(take_name)


def save_project(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
res = c4d.documents.SaveProject(document, c4d.SAVEPROJECT_SCENEFILE | c4d.SAVEPROJECT_USEDOCUMENTNAMEASFILENAME | c4d.SAVEPROJECT_DONTFAILONMISSINGASSETS, document.GetDocumentPath(), [], [])
""", return_variable_name='res')


def get_document_file_path(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
document_file_path = document.GetDocumentPath().replace('\\\\', '/')
if document_file_path:
    document_file_path =  document_file_path + '/' + document.GetDocumentName()
""", return_variable_name='document_file_path')


def get_document_name(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
document_name = c4d.documents.GetActiveDocument().GetDocumentName()
""", return_variable_name='document_name')


def is_document_saved(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
is_document_saved = len(c4d.documents.GetActiveDocument().GetDocumentPath()) > 0
""", return_variable_name='is_document_saved')


def is_document_modified(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
is_document_modified = c4d.documents.GetActiveDocument().GetChanged()
""", return_variable_name='is_document_modified')


def get_current_take_name(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
current_take_name = c4d.documents.GetActiveDocument().GetTakeData().GetCurrentTake().GetName()
""", return_variable_name='current_take_name')


def get_c4d_version_str(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
c4d_version = int(c4d.GetC4DVersion() / 1000)

# dealing with subscription versions
# subscription versions are even numbers and non-subscription are odd ones
# subscription version are available from 22 and above
# subscription version are compatible with c4d version i.e. odd number one lower than it and above
if (c4d_version % 2 == 0 and c4d_version >= 22):
    c4d_version = c4d_version - 1

c4d_version_str = 'R{0}'.format(c4d_version)
""", return_variable_name='c4d_version_str')


def get_output_width(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
output_width = int(render_data[c4d.RDATA_XRES])
""", return_variable_name='output_width')


def get_output_height(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
output_height = int(render_data[c4d.RDATA_YRES])
""", return_variable_name='output_height')


def get_output_format(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
output_formats = {}
output_formats[c4d.FILTER_AVI] = 'AVI'
output_formats[c4d.FILTER_BMP] = 'BMP'
output_formats[c4d.FILTER_B3D] = 'B3D'
output_formats[c4d.FILTER_EXR] = 'EXR'
output_formats[c4d.FILTER_HDR] = 'HDR'
output_formats[c4d.FILTER_IES] = 'IES'
output_formats[c4d.FILTER_IFF] = 'IFF'
output_formats[c4d.FILTER_JPG] = 'JPG'
output_formats[c4d.FILTER_PNG] = 'PNG'
output_formats[c4d.FILTER_PICT] = 'PICT'
output_formats[c4d.FILTER_PSD] = 'PSD'
output_formats[c4d.FILTER_RLA] = 'RLA'
output_formats[c4d.FILTER_RPF] = 'RPF'
output_formats[c4d.FILTER_TGA] = 'TGA'
output_formats[c4d.FILTER_TIF] = 'TIFF'

rdata_format = c4d.documents.GetActiveDocument().GetActiveRenderData()[c4d.RDATA_FORMAT]
output_format = output_formats.get(rdata_format, '')
""", return_variable_name='output_format')

def get_output_multipass_format(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
output_formats = {}
output_formats[c4d.FILTER_AVI] = 'AVI'
output_formats[c4d.FILTER_BMP] = 'BMP'
output_formats[c4d.FILTER_B3D] = 'B3D'
output_formats[c4d.FILTER_EXR] = 'EXR'
output_formats[c4d.FILTER_HDR] = 'HDR'
output_formats[c4d.FILTER_IES] = 'IES'
output_formats[c4d.FILTER_IFF] = 'IFF'
output_formats[c4d.FILTER_JPG] = 'JPG'
output_formats[c4d.FILTER_PNG] = 'PNG'
output_formats[c4d.FILTER_PICT] = 'PICT'
output_formats[c4d.FILTER_PSD] = 'PSD'
output_formats[c4d.FILTER_RLA] = 'RLA'
output_formats[c4d.FILTER_RPF] = 'RPF'
output_formats[c4d.FILTER_TGA] = 'TGA'
output_formats[c4d.FILTER_TIF] = 'TIFF'

multipass_save_format = c4d.documents.GetActiveDocument().GetActiveRenderData()[c4d.RDATA_MULTIPASS_SAVEFORMAT]
output_multipass_format = output_formats.get(multipass_save_format, '')
""", return_variable_name='output_multipass_format')


def get_frame_range_str(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()

fps = document.GetFps()
frame_from = render_data[c4d.RDATA_FRAMEFROM].GetFrame(fps)
frame_to = render_data[c4d.RDATA_FRAMETO].GetFrame(fps)
frame_step = render_data[c4d.RDATA_FRAMESTEP]

frame_range_str = '{} {} {}'.format(frame_from, frame_to, frame_step)
""", return_variable_name='frame_range_str')


def get_global_save(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
global_save = bool(render_data[c4d.RDATA_GLOBALSAVE])
""", return_variable_name='global_save')


def should_save_regular_image(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
save_regular_image = bool(render_data[c4d.RDATA_SAVEIMAGE])
""", return_variable_name='save_regular_image')


def get_render_path(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
render_path = render_data[c4d.RDATA_PATH]
""", return_variable_name='render_path')


def is_global_multipass_enabled(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
is_global_multipass_enabled = bool(render_data[c4d.RDATA_MULTIPASS_ENABLE])
""", return_variable_name='is_global_multipass_enabled')


def is_multipass_save_enabled(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
is_multipass_save_enabled = bool(render_data[c4d.RDATA_MULTIPASS_SAVEIMAGE])
""", return_variable_name='is_multipass_save_enabled')


def get_multipass_render_path(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
data_mp_filename = render_data[c4d.RDATA_MULTIPASS_FILENAME]
""", return_variable_name='data_mp_filename')


def get_render_engine(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
renderers = {
    c4d.RDATA_RENDERENGINE_STANDARD: 'standard',
    c4d.RDATA_RENDERENGINE_PHYSICAL: 'physical'
}

RDATA_RENDERENGINE_ARNOLD = 1029988
RDATA_RENDERENGINE_REDSHIFT = 1036219
RDATA_RENDERENGINE_LEGACY_VRAYBRIDGE = 1019782
RDATA_RENDERENGINE_VRAY = 1053272

renderers[RDATA_RENDERENGINE_REDSHIFT] = 'redshift'
renderers[RDATA_RENDERENGINE_ARNOLD] = 'arnold'
renderers[RDATA_RENDERENGINE_LEGACY_VRAYBRIDGE] = 'vray'
renderers[RDATA_RENDERENGINE_VRAY] = 'vray'
    
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
renderer = render_data[c4d.RDATA_RENDERENGINE]

render_engine = renderers.get(renderer, '')
""", return_variable_name='render_engine')


def save_document_with_assets(cps: 'CommandPortServer', target_path: str):
    target_path = target_path.replace('\\', '\\\\')

    return cps.eval("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()

flags = c4d.SAVEPROJECT_DONTTOUCHDOCUMENT | c4d.SAVEPROJECT_SCENEFILE | c4d.SAVEPROJECT_ASSETS | c4d.SAVEPROJECT_USEDOCUMENTNAMEASFILENAME | c4d.SAVEPROJECT_WITHCACHES

assets = []
missing_assets = []
result = c4d.documents.SaveProject(document, flags, '{}', assets=assets, missingAssets=missing_assets)
""".format(target_path), return_variables=['result', 'assets', 'missing_assets'])


def get_takes(cps: 'CommandPortServer') -> typing.Dict:
    return cps.eval_cmd("""   
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()
main_take = document.GetTakeData().GetMainTake()

takes_to_process = [main_take]
takes = []

while takes_to_process:
    take = takes_to_process.pop(0)
    takes.append({'name': take.GetName(), 'is_checked': take.IsChecked()})
    takes_to_process.extend(take.GetChildren())

""", return_variable_name='takes')


def get_arnold_render_device(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data =document.GetActiveRenderData()
video_post = render_data.GetFirstVideoPost()

render_device = 0 # default to cpu
while video_post:
    if video_post.GetType() == 1029988:
        render_device = video_post[c4d.C4DAIP_OPTIONS_RENDER_DEVICE]
    video_post = video_post.GetNext()

""", return_variable_name='render_device')


def c4d_vray_requires_gpu(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
document = c4d.documents.GetActiveDocument()
render_data = document.GetActiveRenderData()

renderer_type_id = render_data[c4d.RDATA_RENDERENGINE]

vray_requires_gpu = False
video_post = render_data.GetFirstVideoPost()
while video_post:
    if video_post.GetType() == renderer_type_id:
        # RDATA_RENDERENGINE_LEGACY_VRAYBRIDGE
        if renderer_type_id == 1019782:
            # use RT engine for production
            is_use_rt = video_post[c4d.VP_VRAYBRIDGE_USE_RT] == 1
        
            # GPU 1 - OpenCL single Kernel, 4 - CUDA Single Kernel
            use_rt_gpu_engine = video_post[c4d.VP_VRAYBRIDGE_RT_USE_OPENCL] in [1, 4]
        
            vray_requires_gpu = is_use_rt and use_rt_gpu_engine
        elif renderer_type_id == 1053272:
            # Value in UI is V-Ray GPU
            vray_requires_gpu = video_post[c4d.VRAY_VP_VRAY_RENDER_ENGINE] == 1
            
        break
    video_post = video_post.GetNext()

""", return_variable_name='vray_requires_gpu')