__all__ = 'C4DSceneExporter'

import tempfile
import pathlib

from gm_meta_plugin.errors import FilePackingError
from gm_meta_plugin.packed_project import PackedProject
from gm_meta_plugin.scene_exporter import SceneExporter
from . import c4d_constants, c4d_cpc


class C4DSceneExporter(SceneExporter):

    def __init__(self, plugin):
        SceneExporter.__init__(self, plugin)

    def export(self, output_dir: pathlib.Path) -> pathlib.Path:
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()
        logger = plugin.get_logging_coordinator().get_logger(__name__)

        output_dir_str = output_dir.as_posix()
        logger.info("Saving project with assets to folder: '{0}'".format(output_dir_str))

        results = c4d_cpc.save_document_with_assets(cps, output_dir_str)
        assets = results['assets']
        missing_assets = results['missing_assets']
        result = results['result']

        if missing_assets:
            for missing_asset in missing_assets:
                logger.error("Missing asset - '{}'".format(str(missing_asset)))

            msg = "Warning - Your scene contains missing assets.\n" \
                  "Please fix the following missing assets and then retry:\n"

            counter = 0
            while missing_assets and counter < 5:
                missing_asset = missing_assets.pop()
                counter = counter + 1

                msg = msg + "   " + str(missing_asset.get('filename', '<no file name>')) + "\n"

            if missing_assets:
                msg = msg + "   ({0} assets not shown)\n".format(len(missing_assets))

            msg = msg + "\nUse 'Mark Missing Textures' button in Project Info->Textures tab to identify the missing ones. You can debug the issue by running 'File -> Save Project with Assets...' in C4D since this what the GridMarkets plug-in is doing behind the scenes."

            raise FilePackingError(msg)

        if not result:
            raise FilePackingError("Unable to pack project for submission.\nCinema 4D Failed to save your project with assets (via 'File -> Save Project with Assets...') for an unkown reason.")

        logger.info(u"Packed scene to: '{0}'".format(output_dir_str))

        return output_dir
