import typing

if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import CommandPortServer


def get_max_file_name(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
max_file_name = pymxs.runtime.maxFileName 
""", return_variable_name='max_file_name')


def get_max_file_path(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
max_file_path = pymxs.runtime.maxFilePath 
""", return_variable_name='max_file_path')


def get_max_current_renderer(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
max_current_renderer = str(pymxs.runtime.renderers.current)
""", return_variable_name='max_current_renderer')


def is_save_required(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
max_save_required = pymxs.runtime.getSaveRequired()
""", return_variable_name='max_save_required')


def save_current_scene(cps: 'CommandPortServer') -> bool:
    return cps.eval_cmd("""
import os
max_file_path = pymxs.runtime.maxFilePath
max_file_name = pymxs.runtime.maxFileName
return_code = pymxs.runtime.saveMaxFile(os.path.join(max_file_path, max_file_name), clearNeedSaveFlag=True, quiet=True)
""", return_variable_name='return_code')


def zip_scene(cps: 'CommandPortServer', target_file: str, zip_output_path: str):
    target_file = target_file.replace('\\', '\\\\')
    zip_output_path = zip_output_path.replace('\\', '\\\\')

    return cps.eval("""
pymxs.runtime.maz('{}', '{}') 
""".format(zip_output_path, target_file))


def unzip_scene(cps: 'CommandPortServer', zipped_file: str, output_dir: str):
    zipped_file = zipped_file.replace('\\', '\\\\')
    output_dir = output_dir.replace('\\', '\\\\')

    return cps.eval("""
pymxs.runtime.unmaz('{}', '{}')  
""".format(zipped_file, output_dir))


def save_max_scene_file(cps: 'CommandPortServer', save_file_target: str):
    save_file_target = save_file_target.replace('\\', '\\\\')

    return cps.eval("""
pymxs.runtime.saveMaxFile('{}')
""".format(save_file_target))


def load_max_scene_file(cps: 'CommandPortServer', file_path: str):
    file_path = file_path.replace('\\', '\\\\')

    return cps.eval("""
pymxs.runtime.loadMaxFile('{}')
""".format(file_path))


def get_release_version(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
max_release_version = str(pymxs.runtime.maxVersion()[0])  
""", return_variable_name='max_release_version')


def get_camera_names(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
scene_camera_names = []
for camera in pymxs.runtime.cameras:
    # filter out target cameras
    if 'Target' not in camera.name:
        scene_camera_names.append(camera.Name)
""", return_variable_name='scene_camera_names')


def get_current_project_folder(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
current_project_folder = pymxs.runtime.pathConfig.getCurrentProjectFolder()
""", return_variable_name='current_project_folder')


def get_file_in_gamma(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
file_in_gamma = pymxs.runtime.fileInGamma
""", return_variable_name='file_in_gamma')


def get_file_out_gamma(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
file_out_gamma = pymxs.runtime.fileOutGamma
""", return_variable_name='file_out_gamma')


def get_frame_range_str(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
frame_range = ""
start_frame = int(pymxs.runtime.rendStart)
end_frame = int(pymxs.runtime.rendEnd)
frame_step = pymxs.runtime.rendNThFrame
frame_range = "{0} {1} {2}".format(start_frame, end_frame, frame_step)
""", return_variable_name='frame_range')


def get_output_width(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
output_width = pymxs.runtime.renderWidth
""", return_variable_name='output_width')


def get_output_height(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
output_height = pymxs.runtime.renderHeight
""", return_variable_name='output_height')


def get_render_output(cps: 'CommandPortServer'):
    return cps.eval_cmd("""
render_output = pymxs.runtime.getDir(pymxs.runtime.Name("renderoutput"))
""", return_variable_name='render_output')


def get_render_output_path(cps: 'CommandPortServer') -> str:
    return cps.eval_cmd("""
render_output_path = pymxs.runtime.rendOutputFilename
""", return_variable_name='render_output_path')
