__all__ = 'MaxSceneExporter'
import os
import pathlib

import gm_meta_plugin
from . import max_cpc


class MaxSceneExporter(gm_meta_plugin.SceneExporter):

    def export(self, output_dir: pathlib.Path) -> pathlib.Path:
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()
        log = plugin.get_logging_coordinator().get_logger(__name__)

        scene_name = max_cpc.get_max_file_name(cps)
        scene_path = max_cpc.get_max_file_path(cps)

        if not scene_name:
            scene_name = "max_exported_scene.max"

        file_path = os.path.join(scene_path, scene_name)

        output_dir_str = str(output_dir)
        zip_output_path = os.path.join(output_dir_str, "archive.zip")
        packed_dir = os.path.join(output_dir_str, "packed_dir")

        if not os.path.exists(packed_dir):
            os.makedirs(packed_dir)

        log.info(u"Archiving scene to: " + zip_output_path)

        max_cpc.zip_scene(cps, file_path, zip_output_path)
        max_cpc.unzip_scene(cps, zip_output_path, packed_dir)

        log.info(u"Packed scene to: " + packed_dir)
        return pathlib.Path(packed_dir)
