__all__ = 'MayaJobPresetValidator'

import pathlib
import typing

from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_meta_plugin import JobPresetValidator


class MayaJobPresetValidator(JobPresetValidator):

    def validate(self, job_preset) -> typing.Tuple[bool, str]:
        is_valid = True
        validation_message = u""

        render_layers_attribute = job_preset.get_job_preset_attribute_by_key(api_constants.API_KEYS.RENDER_LAYERS)
        if not render_layers_attribute.should_skip():
            render_layers = render_layers_attribute.eval()
            if not render_layers:
                return False, "No render layers enabled.\nYou must have at least one render layer enabled."

        # Validate the output prefix
        output_path_attribute = job_preset.get_job_preset_attribute_by_key(api_constants.API_KEYS.OUTPUT_PATH)

        if output_path_attribute.is_active() and output_path_attribute.is_enabled():
            output_path: str = output_path_attribute.eval()
            error_message_suffix = "in the Submit2gm submission interface."
        else:
            output_path = output_path_attribute.eval_application_defined_value()
            error_message_suffix = "in 'File > Project Window > Images' in Maya."

        if output_path:
            # Check that the output path is relative
            if pathlib.Path(output_path).is_absolute() or output_path.startswith('/'):
                return False, "Output path must be a relative file path.\n" \
                              "Please set a relative value {}".format(error_message_suffix)

            # Check that the output path is within the project folder
            if output_path.startswith('..'):
                return False, "Output path must not start with '..' otherwise results may be written outside of the project folder.\n" \
                              "Please modify the value {}".format(error_message_suffix)

        return is_valid, validation_message
