__all__ = 'NukeAttributeSource'

import pathlib
import typing

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_meta_plugin.errors import ApplicationAttributeNotFound
from gm_meta_plugin.attribute_types import EnumItem
from . import nuke_cpc


if typing.TYPE_CHECKING:
    from submit2gm.submit2gm_plugin import Submit2gmPlugin


class NukeAttributeSource(gm_meta_plugin.ApplicationAttributeSource):

    def __init__(self, plugin: 'Submit2gmPlugin'):
        gm_meta_plugin.ApplicationAttributeSource.__init__(self, plugin)

    def get_enum_items(self, app, version, key):
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()

        if key == api_constants.API_KEYS.APP_VERSION:
            items = []

            current_nuke_version = nuke_cpc.get_version_string(cps)
            items.append(EnumItem(current_nuke_version, current_nuke_version, u""))

            return items, 0

        if key == api_constants.API_KEYS.MACHINE_TYPE:
            items = list()

            api_client = self.get_plugin().get_api_client()
            machines = api_client.get_machines(api_constants.PRODUCTS.NUKE, api_constants.OPERATION_TYPES.RENDER)

            index = 0
            default = 0
            for machine in machines:
                if not machine.is_gpu_machine():
                    items.append(EnumItem(machine.get_id(), machine.get_name(), u""))

                    if machine.is_default():
                        default = index

                    index = index + 1

            return items, default

        raise ApplicationAttributeNotFound(app, version, key)

    def get_attribute_value(self, app, app_version, key):
        plugin = self.get_plugin()
        cps = plugin.get_active_command_port_server()

        if key == api_constants.API_KEYS.IS_SAVED:
            file_name = nuke_cpc.get_script_name(cps)
            return len(file_name) > 0

        if key == api_constants.API_KEYS.PROJECT_NAME:
            return pathlib.Path(nuke_cpc.get_project_dir(cps)).stem

        if key == api_constants.API_KEYS.SUBMISSION_NAME:
            file_name = pathlib.Path(nuke_cpc.get_script_name(cps)).name

            if not file_name:
                return "Untitled"

            return file_name

        if key == api_constants.API_KEYS.JOB_NAME:
            return 'Nuke job'

        if key == api_constants.API_KEYS.APP:
            return api_constants.PRODUCTS.NUKE

        if key == api_constants.API_KEYS.APP_VERSION:
            return nuke_cpc.get_version_string(cps)

        if key == api_constants.API_KEYS.SCENE_FILE_PATH_UNUSED:
            file_path = nuke_cpc.get_script_name(cps)
            return file_path

        if key == api_constants.API_KEYS.FRAMES:
            return nuke_cpc.get_frame_range_str(cps)

        raise ApplicationAttributeNotFound(app, app_version, key)
