__all__ = 'ThreadedUnixStreamServer'

import json
import time

from socketserver import UnixStreamServer, StreamRequestHandler, ThreadingMixIn
from submit2gm import constants
from submit2gm.worker_job import WorkerJob


class Handler(StreamRequestHandler):
    def handle(self):
        current_message = b''
        while True:
            message = b''.join(self.rfile.readlines())

            if not message:
                break

            split_message = message.split(constants.WIN_PIPE_MESSAGE_SEPARATOR)

            for index, value in enumerate(split_message):
                current_message = current_message + value

                # The last item in the split message is a special case.
                if index >= len(split_message) - 1:
                    continue

                argv = json.loads(current_message.decode('utf8'))
                current_message = b''

                job = WorkerJob(argv)
                self.server._job_queue.put(job)

                while not job.is_completed():
                    time.sleep(0.25)


class ThreadedUnixStreamServer(ThreadingMixIn, UnixStreamServer):

    def __init__(self, address: str, job_queue: 'queue.Queue[WorkerJob]'):
        UnixStreamServer.__init__(self, address, Handler)
        self._job_queue = job_queue
